/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2009,2010 by Luigi Ziviani <ziviani_luigi@yahoo.it>.        *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: mimetypedialog.cpp,   v 2.8                      2008/01/01               *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/

#include <xincs.h>
#include <fx.h>
#include "MimeTypeDialog.h"
#include "LanguageDialog.h"
#include "fxmoviemanager.h"
#include "icons.h"
#include "config.h"


FXDEFMAP(MimeTypeDialog) MimeTypeDialogMap[]={

  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONPATH,			MimeTypeDialog::onCmdSelectIconPath),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_COMMAND,			MimeTypeDialog::onCmdSelectCommand),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_MIMETYPE,			MimeTypeDialog::onCmdSelectMimeType),
  FXMAPFUNC(SEL_CLOSE,	MimeTypeDialog::ID_FXMMAPPLICATION,			MimeTypeDialog::onCmdCloseDialog),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_CREATE_MIMEASSOCIATION,	MimeTypeDialog::onCmdCreateFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_DELETE_MIMEASSOCIATION,	MimeTypeDialog::onCmdDeleteFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_RENAME_MIMEASSOCIATION,	MimeTypeDialog::onCmdRenameFileAssociation),
  FXMAPFUNC(SEL_CHANGED,MimeTypeDialog::ID_SELECT_MIMEASSOCIATION,	MimeTypeDialog::onCmdFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_TERMINAL,			MimeTypeDialog::onCmdSelectTerminal),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_FONT,				MimeTypeDialog::onCmdSelectFont),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_FONT2,			MimeTypeDialog::onCmdSelectFont2),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_RESTORE_SETTINGS,		MimeTypeDialog::onCmdRestoreDefaults),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_CUSTOM_LANGUAGE,			MimeTypeDialog::onCmdCustomLanguage),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_THEME_DARK,		MimeTypeDialog::ID_SELECT_THEME_SUN,		MimeTypeDialog::onCmdSelectThemeColors),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_LANGUAGE_EN,		MimeTypeDialog::ID_SELECT_LANGUAGE_CU,		MimeTypeDialog::onCmdSelectLanguage),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONBIG_NAME,	MimeTypeDialog::ID_SELECT_ICONSMALL_NAME,	MimeTypeDialog::onCmdSelectIcon),
  };


// Object implementation
FXIMPLEMENT(MimeTypeDialog,FXDialogBox,MimeTypeDialogMap,ARRAYNUMBER(MimeTypeDialogMap))


// Construct window
MimeTypeDialog::MimeTypeDialog(FXWindow *own):FXDialogBox(own,"FXMovieManager Setup",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0){

  // Default font
  bodyfont=NULL;
  menufont=NULL;

  // get iconpath, theme and language Settings
  fxmmwheellines = getApp()->reg().readIntEntry("SETTINGS","fxmmwheellines",4);
  iconpath = getApp()->reg().readStringEntry("SETTINGS","mimeiconpath");
  theme = getApp()->reg().readStringEntry("SETTINGS","theme");
  language = getApp()->reg().readStringEntry("SETTINGS","language");
  terminal = getApp()->reg().readStringEntry("SETTINGS","terminal");

  fontspec = getApp()->reg().readStringEntry("SETTINGS","bodyfont","");
  if(!fontspec.empty()){
    bodyfont=new FXFont(getApp(),fontspec);
    }

  fontspec2 = getApp()->reg().readStringEntry("SETTINGS","menufont","");
  if(!fontspec2.empty()){
    menufont=new FXFont(getApp(),fontspec2);
    getApp()->setNormalFont(menufont);
    }

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  // get thumbnailer options
  thumbnailSize = getApp()->reg().readIntEntry("THUMBNAILER","thumbnailSize",128);
  imageQuality = getApp()->reg().readIntEntry("THUMBNAILER","imageQuality",8);
  filmStripOverlay = getApp()->reg().readBoolEntry("THUMBNAILER","filmStripOverlay",FALSE);
  workaroundIssues = getApp()->reg().readBoolEntry("THUMBNAILER","workaroundIssues",FALSE);
  maintainAspectRatio = getApp()->reg().readBoolEntry("THUMBNAILER","maintainAspectRatio",TRUE);
  smartFrameSelection = getApp()->reg().readBoolEntry("THUMBNAILER","smartFrameSelection",FALSE);
#endif
  autoThumbnailShow = getApp()->reg().readBoolEntry("THUMBNAILER","autoThumbnailShow",TRUE);
  autoThumbnailCreate = getApp()->reg().readBoolEntry("THUMBNAILER","autoThumbnailCreate",FALSE);
  autoThumbnailUpdate = getApp()->reg().readBoolEntry("THUMBNAILER","autoThumbnailUpdate",FALSE);

  imagethumbnailSize = getApp()->reg().readIntEntry("THUMBNAILER","imagethumbnailSize",128);

  // Set the Main Application Target
  setTarget(this);

  // Connect Datatargets
  target_association_type.connect(fileassociation.type);
  target_association_command.connect(fileassociation.command);
  target_association_mimetype.connect(fileassociation.mimetype);
  target_iconpath.connect(iconpath);
  target_terminal.connect(terminal);
  target_wheel.connect(fxmmwheellines);

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  target_thumbnailSize.connect(thumbnailSize);
  target_imageQuality.connect(imageQuality);
  target_filmStripOverlay.connect(filmStripOverlay);
  target_workaroundIssues.connect(workaroundIssues);
  target_maintainAspectRatio.connect(maintainAspectRatio);
  target_smartFrameSelection.connect(smartFrameSelection);
#endif
  target_autoThumbnailShow.connect(autoThumbnailShow);
  target_autoThumbnailCreate.connect(autoThumbnailCreate);
  target_autoThumbnailUpdate.connect(autoThumbnailUpdate);

  target_imagethumbnailSize.connect(imagethumbnailSize);

  // Set the message identifier for this data target
  setSelector(ID_FXMMAPPLICATION);

  // Retrive theme settings
  theme=getApp()->reg().readStringEntry("SETTINGS","theme");
  theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor");
  theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor");
  theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor");
  theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor");
  theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor");
  theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor");
  theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor");
  theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor");
  theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor");
  theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor");
  theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor");
  theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor");

  // Build icons
  logoicon=new FXPNGIcon(getApp(),fxmm64);
  abouticon=new FXGIFIcon(getApp(),b_list);
  conficon=new FXGIFIcon(getApp(),b_conf);
  diricon=new FXGIFIcon(getApp(),m_dir);
  fonticon=new FXGIFIcon(getApp(),m_renamedir);
  cancelicon=new FXGIFIcon(getApp(),m_cancel);
  accepticon=new FXGIFIcon(getApp(),m_accept);
  execicon=new FXGIFIcon(getApp(),m_console);

  darkthemeicon=new FXGIFIcon(getApp(),darktheme);
  shinethemeicon=new FXGIFIcon(getApp(),shinetheme);
  classicthemeicon=new FXGIFIcon(getApp(),classictheme);
  gentlethemeicon=new FXGIFIcon(getApp(),gentletheme);
  fogthemeicon=new FXGIFIcon(getApp(),fogtheme);
  moonthemeicon=new FXGIFIcon(getApp(),moontheme);
  pinkthemeicon=new FXGIFIcon(getApp(),pinktheme);
  bloodthemeicon=new FXGIFIcon(getApp(),bloodtheme);

  en_flagicon=new FXGIFIcon(getApp(),en_flag);
  it_flagicon=new FXGIFIcon(getApp(),it_flag);
  de_flagicon=new FXGIFIcon(getApp(),de_flag);
  fr_flagicon=new FXGIFIcon(getApp(),fr_flag);
  es_flagicon=new FXGIFIcon(getApp(),es_flag);
  ru_flagicon=new FXGIFIcon(getApp(),ru_flag);
  cn_flagicon=new FXGIFIcon(getApp(),cn_flag);
  jp_flagicon=new FXGIFIcon(getApp(),jp_flag);

  // get the size of normal font and create a new font with size x 2

  getApp()->getNormalFont()->create();

  FXFontDesc fontdescription=getApp()->getNormalFont()->getFontDesc();
  menufont = new FXFont(getApp(),fontdescription);
  menufont->create();

  fontdescription.size = (FXuint) (((double)fontdescription.size) * 2.0);
  titlefont = new FXFont(getApp(),fontdescription);
  titlefont->create();

  languagedialog=new LanguageDialog(this);

  tooltip=new FXToolTip(getApp());

  // Declare reused frames
  FXHorizontalFrame *hframe=NULL;
  FXVerticalFrame   *sframe=NULL;
  FXVerticalFrame   *vframe=NULL;
  FXLabel           *label=NULL;
  FXPacker 	    	*packer;

  // main frame
  FXVerticalFrame   *mainframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  // Get icon margin
  const FXint spacing=(logoicon ? 10 : 20);

  // Create the header
  label=new FXLabel(mainframe," FXMovieManager Configuration",logoicon,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,spacing,spacing,spacing,spacing);
  label->setBackColor(FXRGB(0,0,0));
  label->setTextColor(FXRGB(255,255,0));
  label->setFont(titlefont);

  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  
  FXHorizontalFrame *hmainframe=new FXHorizontalFrame(mainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  FXVerticalFrame *buttonframe=new FXVerticalFrame(hmainframe,LAYOUT_FILL_Y|LAYOUT_LEFT|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT,0,0,0,0, 5,5,5,5);
  new FXSeparator(hmainframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);


  FXSwitcher *switcher = new FXSwitcher(hmainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);

  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"About",abouticon,switcher,FXSwitcher::ID_OPEN_FIRST,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"HowTo",abouticon,switcher,FXSwitcher::ID_OPEN_SECOND,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Keyboard",abouticon,switcher,FXSwitcher::ID_OPEN_THIRD,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"MimeTypes",conficon,switcher,FXSwitcher::ID_OPEN_FOURTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Configure",conficon,switcher,FXSwitcher::ID_OPEN_FIFTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Thumbnail",conficon,switcher,FXSwitcher::ID_OPEN_SIXTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);


  // About this program
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," About FXMovieManager",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	"FXMovieManager is a FileManager with movie thumbnails,\n"
	"it manages a PlayList and act as simple front-end to MPlayer.\n"
	"Copyright (C) 2008,2010 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"
	);



  // Help instructions
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," How to use FXMovieManager",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	" To manage thumbnails of movie files a list of buttons on the\n"
	" right of the main application area can be used.\n\n"
	" 1)  To Create and Show thumbnails of movie files:\n"
	" select the movies and press the button [Create thumbnails].\n"
	" 2)  To ReCreate the thumbnails:\n"
	" select the movies and press the button [Recreate thumbnails].\n"
	" 3)  To load a movie into the playlist:\n"
	" double click on a movie in the filelist or open the file dialog\n"
	" and select one or more movies keeping [Ctrl] or [Shift] pressed.\n"
	" 4)  To delete hidden directories that store the thumbnails:\n"
	" press the button [Delete Thumbnails]\n"
	" 5)  To associate file extensions to icons, mime-types and commands:\n"
	" Select the button [MimeTypes] in the Current dialog and\n"
	" a window will be displayed where to add new file associations\n"
	" 6)  To change Theme/Language/Font/Terminal:\n"
	" Select the button  [Configure] in the Current dialog and\n"
	" a window will be displayed where to set your preferences\n"
	" 7)  To manages files: right mouse click on the file/files selected\n"
	" 8)  To restore default settings click on the button below\n"
	,NULL,JUSTIFY_LEFT);

  button_restore = new FXButton(vframe,"Restore Default Settings\tRestore predefined settings",execicon,this,ID_RESTORE_SETTINGS,FRAME_RAISED|ICON_BEFORE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,260,32, 1,1,1,1);

  // Short Cut instructions
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," Some keyboard controls are enabled",NULL,JUSTIFY_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  label = new FXLabel(vframe,
	"     F1 = show help and configuration dialog\n"
	" Other Function buttons: CTRL manages LEFT Panel\n"
	"     F2 = reverse file selection\n"
	"     F3 = create thunbnails of selected files\n"
	"     F4 = force thumbnails recreation of selected files\n"
	"     F5 = increase thumbnails size of selected files\n"
	"     F6 = decrease thumbnails size of selected files\n"
	"     F7 = show big icon list\n"
	"     F8 = show small icon list\n"
	" Keys: CTRL manages LEFT Panel\n"
	"     Delete = delete selected files\n"
	" Other Keys: CTRL manages LEFT Panel   ALT manages RIGHT Panel\n"
	"     C or c = copy\n"
	"     S or s = symlink\n"
	"     R or r = rename\n"
	"     Z or z = play\n"
	"     A or a = pause\n"
	"     Q or q = quit\n" 
	,NULL,JUSTIFY_LEFT);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," Mouse Wheel Speed ",execicon,LAYOUT_LEFT|LAYOUT_CENTER_Y|ICON_BEFORE_TEXT);
  mousespinner=new FXSpinner(hframe,4,&target_wheel,FXDataTarget::ID_VALUE,FRAME_SUNKEN|FRAME_THICK);
  mousespinner->setRange(1,20);


  // Content-Types configuration
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(hframe,"Search Path for Icons:",NULL,LAYOUT_CENTER_Y);
  new FXTextField(hframe,2,&target_iconpath,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,"..",diricon,this,ID_SELECT_ICONPATH,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y,0,0,0,0,5,5,5,5);

  new FXSeparator(hframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);

  new FXLabel(vframe,"Extensions of File: ");
  sframe = new FXVerticalFrame(vframe,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  associationlist = new FXList(sframe,this,ID_SELECT_MIMEASSOCIATION,LAYOUT_FILL_Y|LAYOUT_FILL_X|LIST_BROWSESELECT);
  associationlist->setSortFunc(FXList::ascending);
  FXHorizontalFrame * listbuttonframe = new FXHorizontalFrame(vframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  new FXButton(listbuttonframe,"&Insert\tInsert a New Extension",NULL,this,ID_CREATE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Delete\tDelete an Extension",NULL,this,ID_DELETE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Change\tChange the name of an Extension",NULL,this,ID_RENAME_MIMEASSOCIATION);

  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Type Name:");
  new FXTextField(packer,2,&target_association_type,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Content-Type [object/format]:");
  mimetypelist = new FXComboBox(packer,1,this,ID_SELECT_MIMETYPE,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|COMBOBOX_NO_REPLACE);
  mimetypelist->setSortFunc(FXList::ascending);
  mimetypelist->setNumVisible(9);
  mimetypelist->appendItem(" ");

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Open With Command:");
  new FXButton(packer,"..",diricon,this,ID_SELECT_COMMAND,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  new FXTextField(packer,2,&target_association_command,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  FXMatrix *iconsmatrix=new FXMatrix(vframe,4,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  new FXLabel(iconsmatrix,"Normal",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_bigname = new FXButton(iconsmatrix,"\tChange Normalsize MIME-Type Icon",NULL,this,ID_SELECT_ICONBIG_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);
  new FXLabel(iconsmatrix,"Small",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_smallname = new FXButton(iconsmatrix,"\tChange Smallsize MIME-Type Icon",NULL,this,ID_SELECT_ICONSMALL_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);


  // import file associations **********************************************//
  FXString datastring;
  FXString mimetype;

  FXStringDict *filetypes; // The dictionary class maintains a fast-access 
                           // hash table of entities indexed by a character string.

  filetypes = getApp()->reg().find("FILETYPES"); // FILETYPES is the registry section where are contents-type data

  if(filetypes){
    for(FXint i=filetypes->first(); i<filetypes->size(); i=filetypes->next(i)){
      associationlist->appendItem(filetypes->key(i),NULL,NULL,TRUE); 	// Key string of struct FXDictEntry
      datastring = filetypes->data(i); 					// Data of struct FXDictEntry
      mimetype = datastring.section(";",4);
      if (!mimetype.empty() && (mimetypelist->findItem(mimetype)==-1)){
        mimetypelist->appendItem(mimetype);
        }
      }
    }

  associationlist->sortItems();
  mimetypelist->sortItems();
  //***********************************************************************//



  // Preferences
  FXMatrix *framematrix=new FXMatrix(switcher,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(framematrix," Theme selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXLabel(framematrix," Language selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);


  FXMatrix *themesmatrix=new FXMatrix(framematrix,3,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  darkthemebutton=new FXButton(themesmatrix,"\tDark Theme",NULL,this,ID_SELECT_THEME_DARK,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  darkthemebutton->setIcon(darkthemeicon);
  darkthemebutton->setBackColor(FXRGB(60, 60, 60));
  gentlethemebutton=new FXButton(themesmatrix,"\tStorm Theme",NULL,this,ID_SELECT_THEME_STORM,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  gentlethemebutton->setIcon(gentlethemeicon);
  gentlethemebutton->setBackColor(FXRGB(110,110,110));
  gentlethemebutton=new FXButton(themesmatrix,"\tGentle Theme",NULL,this,ID_SELECT_THEME_GENTLE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  gentlethemebutton->setIcon(gentlethemeicon);
  gentlethemebutton->setBackColor(FXRGB(160,160,160));

  fogthemebutton=new FXButton(themesmatrix,"\tFog Theme",NULL,this,ID_SELECT_THEME_FOG,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  fogthemebutton->setIcon(fogthemeicon);
  fogthemebutton->setBackColor(FXRGB(180,180,180));
  shinethemebutton=new FXButton(themesmatrix,"\tShine Theme",NULL,this,ID_SELECT_THEME_SHINE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  shinethemebutton->setIcon(shinethemeicon);
  shinethemebutton->setBackColor(FXRGB(210,210,210));
  shinethemebutton=new FXButton(themesmatrix,"\tSnow Theme",NULL,this,ID_SELECT_THEME_SNOW,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  shinethemebutton->setIcon(shinethemeicon);
  shinethemebutton->setBackColor(FXRGB(230,230,230));

  bloodthemebutton=new FXButton(themesmatrix,"\tBlood Theme",NULL,this,ID_SELECT_THEME_BLOOD,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  bloodthemebutton->setIcon(bloodthemeicon);
  bloodthemebutton->setBackColor(FXRGB(130, 30, 30));
  pinkthemebutton=new FXButton(themesmatrix,"\tRed Theme",NULL,this,ID_SELECT_THEME_RED,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  pinkthemebutton->setIcon(bloodthemeicon);
  pinkthemebutton->setBackColor(FXRGB(160, 60, 60));
  pinkthemebutton=new FXButton(themesmatrix,"\tPink Theme",NULL,this,ID_SELECT_THEME_PINK,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  pinkthemebutton->setIcon(pinkthemeicon);
  pinkthemebutton->setBackColor(FXRGB(240,180,180));

  moonthemebutton=new FXButton(themesmatrix,"\tMoon Theme",NULL,this,ID_SELECT_THEME_MOON,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  moonthemebutton->setIcon(moonthemeicon);
  moonthemebutton->setBackColor(FXRGB(30, 30, 30));
  classicthemebutton=new FXButton(themesmatrix,"\tClassic Theme",NULL,this,ID_SELECT_THEME_CLASSIC,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  classicthemebutton->setIcon(classicthemeicon);
  classicthemebutton->setBackColor(FXRGB(180,180,180));
  sunthemebutton=new FXButton(themesmatrix,"\tSun Theme",NULL,this,ID_SELECT_THEME_SUN,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  sunthemebutton->setIcon(classicthemeicon);
  sunthemebutton->setBackColor(FXRGB(230,230,230));


  FXMatrix *languagesmatrix=new FXMatrix(framematrix,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  en_button=new FXButton(languagesmatrix,"\tEN language",NULL,this,ID_SELECT_LANGUAGE_EN,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  en_button->setIcon(en_flagicon);
  it_button=new FXButton(languagesmatrix,"\tIT language",NULL,this,ID_SELECT_LANGUAGE_IT,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  it_button->setIcon(it_flagicon);
  de_button=new FXButton(languagesmatrix,"\tDE language",NULL,this,ID_SELECT_LANGUAGE_DE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  de_button->setIcon(de_flagicon);
  fr_button=new FXButton(languagesmatrix,"\tFR language",NULL,this,ID_SELECT_LANGUAGE_FR,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  fr_button->setIcon(fr_flagicon);
  es_button=new FXButton(languagesmatrix,"\tES language",NULL,this,ID_SELECT_LANGUAGE_ES,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  es_button->setIcon(es_flagicon);
  ru_button=new FXButton(languagesmatrix,"\tRU language",NULL,this,ID_SELECT_LANGUAGE_RU,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  ru_button->setIcon(ru_flagicon);
  cn_button=new FXButton(languagesmatrix,"\tCN language",NULL,this,ID_SELECT_LANGUAGE_CN,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  cn_button->setIcon(cn_flagicon);
  jp_button=new FXButton(languagesmatrix,"\tJP language",NULL,this,ID_SELECT_LANGUAGE_JP,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,80, 1,1,1,1);
  jp_button->setIcon(jp_flagicon);
  cu_button=new FXButton(languagesmatrix,"Custom\tCustom language",NULL,this,ID_SELECT_LANGUAGE_CU,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,30, 1,1,1,1);
  cu_setting=new FXButton(languagesmatrix,"Configure\tConfigure Custom Language",NULL,this,ID_CUSTOM_LANGUAGE,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,100,30, 1,1,1,1);

  FXHorizontalFrame *fontframe=new  FXHorizontalFrame(framematrix,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(fontframe,"Files Font",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXLabel(fontframe,"Menus Font",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);

  new FXLabel(framematrix,"Terminal",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);


  FXHorizontalFrame *bfontframe=new  FXHorizontalFrame(framematrix,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  fontbutton = new FXButton(bfontframe,"Click to Select  ",fonticon,this,ID_SELECT_FONT,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  if(!fontspec.empty()){
    fontbutton->setFont(bodyfont);
    }
  fontbutton2 = new FXButton(bfontframe,"Click to Select  ",fonticon,this,ID_SELECT_FONT2,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  if(!fontspec2.empty()){
    fontbutton2->setFont(menufont);
    }

  hframe = new FXHorizontalFrame(framematrix,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXTextField(hframe,2,&target_terminal,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,"..",diricon,this,ID_SELECT_TERMINAL,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);

  //***********************************************************************//



  // Thumbnailer
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
  new FXLabel(vframe," ffmpegthumbnailer options",NULL,JUSTIFY_CENTER_Y);
  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXMatrix *thumbmatrix1=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(thumbmatrix1," thumbnail Size",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXTextField(thumbmatrix1,5,&target_thumbnailSize,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXLabel(thumbmatrix1," image Quality",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  FXRealSpinner *qualityspinner = new FXRealSpinner(thumbmatrix1,2,&target_imageQuality,FXDataTarget::ID_VALUE,REALSPIN_CYCLIC|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
  qualityspinner->setRange(0,11);

  new FXLabel(thumbmatrix1," film Strip Overlay",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_filmStripOverlay,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," workaround Issues",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_workaroundIssues,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," maintain Aspect Ratio",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_maintainAspectRatio,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix1," smart Frame Selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix1,"NO","YES",cancelicon,accepticon,&target_smartFrameSelection,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
#endif


  new FXLabel(vframe," auto Thumbnailer options",NULL,JUSTIFY_CENTER_Y);
  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXMatrix *thumbmatrix2=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(thumbmatrix2," auto Thumbnail Show",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix2,"NO","YES",cancelicon,accepticon,&target_autoThumbnailShow,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix2," auto Thumbnail Create",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix2,"NO","YES",cancelicon,accepticon,&target_autoThumbnailCreate,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXLabel(thumbmatrix2," auto Thumbnail Update",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXToggleButton(thumbmatrix2,"NO","YES",cancelicon,accepticon,&target_autoThumbnailUpdate,FXDataTarget::ID_VALUE,BUTTON_TOOLBAR|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK);
  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  new FXLabel(vframe," Image Preview options",NULL,JUSTIFY_CENTER_Y);
  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXMatrix *thumbmatrix3=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);

  new FXLabel(thumbmatrix3," image Preview Size",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  new FXTextField(thumbmatrix3,5,&target_imagethumbnailSize,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vframe,"autoThumbnailCreate is only available if autoThumbnailShow = TRUE",NULL,TEXTFIELD_INTEGER|LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);

  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXHorizontalFrame *copyright=new FXHorizontalFrame(mainframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  label = new FXLabel(copyright,"FXMovieManager Settings - Copyright (C) Luigi Ziviani",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();

  new FXButton(copyright," &Quit or Save",accepticon,this,FXTopWindow::ID_CLOSE,BUTTON_INITIAL|ICON_BEFORE_TEXT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);

  }


// Delete window
MimeTypeDialog::~MimeTypeDialog(){
  delete titlefont;
  delete bodyfont;
  if(menufont) delete menufont;
  delete logoicon;
  delete abouticon;
  delete conficon;
  delete diricon;
  delete fonticon;
  delete execicon;
  delete accepticon;
  delete cancelicon;
  delete darkthemeicon;
  delete shinethemeicon;
  delete classicthemeicon;
  delete gentlethemeicon;
  delete fogthemeicon;
  delete moonthemeicon;
  delete pinkthemeicon;
  delete bloodthemeicon;
  delete en_flagicon;
  delete it_flagicon;
  delete de_flagicon;
  delete fr_flagicon;
  delete es_flagicon;
  delete ru_flagicon;
  delete cn_flagicon;
  delete jp_flagicon;
  }


// Search for Icon and load it
static FXIcon *createIconApp(FXApp *app,const FXString& name,const FXString& iconpath){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  FXString fullpath;
  if(FXStat::exists(name)){
    fullpath=name;
    }
  else{
    fullpath=FXPath::search(iconpath,name);
    if(fullpath.empty()) return NULL;
    }
  icon=iconsource.loadIconFile(fullpath);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    return icon;
    }
  return NULL;
  }


// Update Icon
static void updateIconApp(FXApp* app,FXButton* button,const FXString& path){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  if(button->getIcon()){
    icon=button->getIcon();
    button->setIcon(NULL);
    delete icon;
    icon=NULL;
    }
  icon=iconsource.loadIconFile(path);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    button->setIcon(icon);
    }
  }


// Save file association entry
void MimeTypeDialog::saveFileAssociation(){
  if(fileassociation.extension.empty()) return;
  FXString added;
  added=fileassociation.command+";"+fileassociation.type+";"+fileassociation.bigname+";"+fileassociation.smallname+";"+fileassociation.mimetype;
  getApp()->reg().writeStringEntry("FILETYPES",fileassociation.extension.text(),added.text());
  }


// Save language
long MimeTypeDialog::onCmdSelectLanguage(FXObject*,FXSelector sel,void*){
    switch(FXSELID(sel)){
	case ID_SELECT_LANGUAGE_EN:
		language="en";
		break;
	case ID_SELECT_LANGUAGE_IT:
		language="it";
		break;
	case ID_SELECT_LANGUAGE_DE:
		language="de";
		break;
	case ID_SELECT_LANGUAGE_FR:
		language="fr";
		break;
	case ID_SELECT_LANGUAGE_ES:
		language="es";
		break;
	case ID_SELECT_LANGUAGE_RU:
		language="ru";
		break;
	case ID_SELECT_LANGUAGE_CN:
		language="cn";
		break;
	case ID_SELECT_LANGUAGE_JP:
		language="jp";
		break;
	case ID_SELECT_LANGUAGE_CU:
		language="cu";
		break;
	}
    return 1;
}


// Save theme colors
long MimeTypeDialog::onCmdSelectThemeColors(FXObject*,FXSelector sel,void*){
    switch(FXSELID(sel)){
	case ID_SELECT_THEME_DARK:
  		theme="dark";
  		theme_fxmmfore   = FXRGB(200,200,200);
  		theme_fxmmbase   = FXRGB( 60, 60, 60);
  		theme_fxmmborder = FXRGB(  0,  0,  0);
  		theme_fxmmback   = FXRGB( 20, 20, 20);
  		theme_fxmmmenufore  = FXRGB(200,200,200);
  		theme_fxmmmenuback  = FXRGB( 10, 10, 10);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(100,100,100);
  		theme_fxmmtipfore  = FXRGB(200,200,200);
  		theme_fxmmtipback  = FXRGB( 20, 20, 20);
  		theme_fxmmhilite  = FXRGB(100,100,100);
  		theme_fxmmshadow  = FXRGB( 10, 10, 10);
	break;
	case ID_SELECT_THEME_STORM:
  		theme="storm";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(110,110,110);
  		theme_fxmmborder = FXRGB( 20, 20, 20);
  		theme_fxmmback   = FXRGB(140,140,140);
  		theme_fxmmmenufore  = FXRGB(220,220,220);
  		theme_fxmmmenuback  = FXRGB(50, 50, 50);
  		theme_fxmmselfore   = FXRGB(200,200,200);
  		theme_fxmmselback   = FXRGB(70, 70, 70);
  		theme_fxmmtipfore  = FXRGB(200,200,200);
  		theme_fxmmtipback  = FXRGB(70,70,70);
  		theme_fxmmhilite  = FXRGB(120,120,120);
  		theme_fxmmshadow  = FXRGB(50,50,50);
	break;
	case ID_SELECT_THEME_GENTLE:
  		theme="gentle";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(160,160,160);
  		theme_fxmmborder = FXRGB( 40, 40, 40);
  		theme_fxmmback   = FXRGB(220,220,220);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(200,200,200);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(180,180,180);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(180,180,180);
  		theme_fxmmshadow  = FXRGB(90,90,90);
	break;

	case ID_SELECT_THEME_FOG:
  		theme="fog";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(180,180,180);
  		theme_fxmmborder = FXRGB( 50, 50, 50);
  		theme_fxmmback   = FXRGB(230,230,230);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(200,200,200);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(180,180,180);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(200,200,200);
  		theme_fxmmshadow  = FXRGB(110,110,110);
	break;
	case ID_SELECT_THEME_SHINE:
  		theme="shine";
  		theme_fxmmfore   = FXRGB(80,80,80);
  		theme_fxmmbase   = FXRGB(210,210,210);
  		theme_fxmmborder = FXRGB(70, 70, 70);
  		theme_fxmmback   = FXRGB(240,240,240);
  		theme_fxmmmenufore  = FXRGB( 30, 30, 30);
  		theme_fxmmmenuback  = FXRGB(190,190,190);
  		theme_fxmmselfore   = FXRGB( 30, 30, 30);
  		theme_fxmmselback   = FXRGB(200,200,200);
  		theme_fxmmtipfore  = FXRGB(70, 70, 70);
  		theme_fxmmtipback  = FXRGB(230,230,230);
  		theme_fxmmhilite  = FXRGB(230,230,230);
  		theme_fxmmshadow  = FXRGB(140,140,140);
	break;
	case ID_SELECT_THEME_SNOW:
  		theme="snow";
  		theme_fxmmfore   = FXRGB(100,100,100);
  		theme_fxmmbase   = FXRGB(230,230,230);
  		theme_fxmmborder = FXRGB( 80, 80, 80);
  		theme_fxmmback   = FXRGB(250,250,250);
  		theme_fxmmmenufore  = FXRGB( 50, 50, 50);
  		theme_fxmmmenuback  = FXRGB(180,180,180);
  		theme_fxmmselfore   = FXRGB( 50, 50, 50);
  		theme_fxmmselback   = FXRGB(220,220,220);
  		theme_fxmmtipfore  = FXRGB(100,100,100);
  		theme_fxmmtipback  = FXRGB(250,250,250);
  		theme_fxmmhilite  = FXRGB(250,250,250);
  		theme_fxmmshadow  = FXRGB(160,160,160);
	break;
	case ID_SELECT_THEME_BLOOD:
  		theme="blood";
  		theme_fxmmfore   = FXRGB( 240,160,160);
  		theme_fxmmbase   = FXRGB(130, 30, 30);
  		theme_fxmmborder = FXRGB( 80, 20, 20);
  		theme_fxmmback   = FXRGB(80, 20, 20);
  		theme_fxmmmenufore = FXRGB( 40,  0,  0);
  		theme_fxmmmenuback = FXRGB(200,100,100);
  		theme_fxmmselfore  = FXRGB( 40,  0,  0);
  		theme_fxmmselback  = FXRGB(180,100,100);
  		theme_fxmmtipfore  = FXRGB( 90, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,160,160);
  		theme_fxmmhilite  = FXRGB(180, 80, 80);
  		theme_fxmmshadow  = FXRGB(80, 20, 20);
	break;
	case ID_SELECT_THEME_RED:
  		theme="red";
  		theme_fxmmfore   = FXRGB(250,180,180 );
  		theme_fxmmbase   = FXRGB(160, 60, 60);
  		theme_fxmmborder = FXRGB( 80, 20, 20);
  		theme_fxmmback   = FXRGB(120, 40, 40);
  		theme_fxmmmenufore = FXRGB( 40,  0,  0);
  		theme_fxmmmenuback = FXRGB(200,100,100);
  		theme_fxmmselfore  = FXRGB( 40,  0,  0);
  		theme_fxmmselback  = FXRGB(180,100,100);
  		theme_fxmmtipfore  = FXRGB( 90, 50, 50);
  		theme_fxmmtipback  = FXRGB(230,180,180);
  		theme_fxmmhilite  = FXRGB(190, 90, 90);
  		theme_fxmmshadow  = FXRGB(100, 40, 40);
	break;
	case ID_SELECT_THEME_PINK:
  		theme="pink";
  		theme_fxmmfore   = FXRGB(140,80,80);
  		theme_fxmmbase   = FXRGB(250,190,190);
  		theme_fxmmborder = FXRGB(120, 80, 80);
  		theme_fxmmback   = FXRGB(250,230,230);
  		theme_fxmmmenufore  = FXRGB( 90, 50, 50);
  		theme_fxmmmenuback  = FXRGB(220,160,160);
  		theme_fxmmselfore   = FXRGB( 90, 50, 50);
  		theme_fxmmselback   = FXRGB(230,180,180);
  		theme_fxmmtipfore  = FXRGB(140,100,100);
  		theme_fxmmtipback  = FXRGB(250,230,230);
  		theme_fxmmhilite  = FXRGB(250,230,230);
  		theme_fxmmshadow  = FXRGB(200,160,160);
	break;
	case ID_SELECT_THEME_MOON:
  		theme="moon";
  		theme_fxmmfore   = FXRGB(150,150,150);
  		theme_fxmmbase   = FXRGB( 30, 30, 30);
  		theme_fxmmborder = FXRGB(  0,  0,  0);
  		theme_fxmmback   = FXRGB(  0,  0,  0);
  		theme_fxmmmenufore  = FXRGB(150,150,150);
  		theme_fxmmmenuback  = FXRGB( 30, 30, 30);
  		theme_fxmmselfore   = FXRGB(10 ,10 , 10);
  		theme_fxmmselback   = FXRGB(100,100,100);
  		theme_fxmmtipfore  = FXRGB(150,150,150);
  		theme_fxmmtipback  = FXRGB( 20, 20, 20);
  		theme_fxmmhilite  = FXRGB(100,100,100);
  		theme_fxmmshadow  = FXRGB( 10, 10, 10);
	break;
	case ID_SELECT_THEME_CLASSIC:
 		theme="classic";
  		theme_fxmmfore   = FXRGB(  0,  0,  0);
  		theme_fxmmbase   = FXRGB(180,180,180);
  		theme_fxmmborder = FXRGB( 50, 50, 50);
  		theme_fxmmback   = FXRGB(210,210,210);
  		theme_fxmmmenufore  = FXRGB(  0,  0,  0);
  		theme_fxmmmenuback  = FXRGB(130,130,130);
  		theme_fxmmselfore   = FXRGB(  0,  0,  0);
  		theme_fxmmselback   = FXRGB(150,150,150);
  		theme_fxmmtipfore  = FXRGB( 50, 50, 50);
  		theme_fxmmtipback  = FXRGB(210,210,210);
  		theme_fxmmhilite  = FXRGB(200,200,200);
  		theme_fxmmshadow  = FXRGB(110,110,110);
	break;
	case ID_SELECT_THEME_SUN:
  		theme="sun";
  		theme_fxmmfore   = FXRGB(100,100,100);
  		theme_fxmmbase   = FXRGB(230,230,230);
  		theme_fxmmborder = FXRGB( 80, 80, 80);
  		theme_fxmmback   = FXRGB(250,250,250);
  		theme_fxmmmenufore  = FXRGB( 50, 50, 50);
  		theme_fxmmmenuback  = FXRGB(180,180,180);
  		theme_fxmmselfore   = FXRGB( 50, 50, 50);
  		theme_fxmmselback   = FXRGB(220,220,220);
  		theme_fxmmtipfore  = FXRGB(100,100,100);
  		theme_fxmmtipback  = FXRGB(250,250,250);
  		theme_fxmmhilite  = FXRGB(250,250,250);
  		theme_fxmmshadow  = FXRGB(160,160,160);
	break;

	}
    return 1;
}


  // Save Colors in the registry file
void MimeTypeDialog::saveThemeColors(){
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbasecolor",theme_fxmmbase);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmforecolor",theme_fxmmfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbackcolor",theme_fxmmback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmbordercolor",theme_fxmmborder);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmmenuforecolor",theme_fxmmmenufore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmmenubackcolor",theme_fxmmmenuback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmselforecolor",theme_fxmmselfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmselbackcolor",theme_fxmmselback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmtipforecolor",theme_fxmmtipfore);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmtipbackcolor",theme_fxmmtipback);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmhilitecolor",theme_fxmmhilite);
  getApp()->reg().writeColorEntry("SETTINGS","fxmmshadowcolor",theme_fxmmshadow);
}


// Set search path for icons
long MimeTypeDialog::onCmdSelectIconPath(FXObject*,FXSelector,void*){
  FXString path=(iconpath.empty()) ? "/usr/local/share/pixmaps" : iconpath.text();
  FXDirDialog opendialog(this,"Search path for icons?");
  opendialog.setDirectory(path);
  if(opendialog.execute()){
    iconpath=opendialog.getDirectory();
    }
  return 1;
  }


// Set terminal
long MimeTypeDialog::onCmdSelectTerminal(FXObject*,FXSelector,void*){
  FXString term=(terminal.empty()) ? "/usr/bin/xterm" : terminal.text();
  FXFileDialog opendialog(this,"Terminal?");
  if(opendialog.execute()){
    terminal=opendialog.getFilename();
    }
  return 1;
  }


// Set command to open the file with
long MimeTypeDialog::onCmdSelectCommand(FXObject*,FXSelector,void*){
  FXString path=(fileassociation.command.empty()) ? "/usr/bin/" : fileassociation.command.text();
  FXString search=FXFileDialog::getOpenFilename(this,"Command to open file with?",path,"*");
  if(!search.empty()){
    fileassociation.command=search;
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectMimeType(FXObject*,FXSelector,void*ptr){
  FXString content=(FXchar*)ptr;
  if(!content.empty() && (mimetypelist->findItem(content)==-1)){
    mimetypelist->appendItem(content);
    mimetypelist->sortItems();
    }
  fileassociation.mimetype=content;
  return 1;
  }


long MimeTypeDialog::onCmdFileAssociation(FXObject*,FXSelector,void*){
  saveFileAssociation();

  FXIcon *newicon;
  FXString iconname;
  FXString association;

  fileassociation.extension=associationlist->getItemText(associationlist->getCurrentItem());
  if(fileassociation.extension.empty()) return 1;

  association=getApp()->reg().readStringEntry("FILETYPES",fileassociation.extension.text());

  // here is the same association order of ControlPanel to be able to import associations 
  fileassociation.command = association.section(";",0);  // the command to open the file with
  fileassociation.type = association.section(";",1);     // the name of the type
  fileassociation.bigname = association.section(";",2);  // the big icon 
  fileassociation.smallname = association.section(";",3); // the small icon 
  fileassociation.mimetype = association.section(";",4); // the content-type [object/format]

  if(!fileassociation.mimetype.empty()){
    FXint no=mimetypelist->findItem(fileassociation.mimetype);
    mimetypelist->setCurrentItem(no);
    }
  else {
    FXint no=mimetypelist->findItem(" ");
    mimetypelist->setCurrentItem(no);
    }

  if(button_bigname->getIcon()){
    newicon=button_bigname->getIcon();
    button_bigname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.bigname,iconpath);
  button_bigname->setIcon(newicon);

  if(button_smallname->getIcon()){
    newicon=button_smallname->getIcon();
    button_smallname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.smallname,iconpath);
  button_smallname->setIcon(newicon);

  return 1;
  }


long MimeTypeDialog::onCmdCreateFileAssociation(FXObject*,FXSelector,void*){
  FXString result;
  if(FXInputDialog::getString(result,this,"New File Association","Please enter fileassociation extension:",NULL)){
    if(result.empty()) return 1;
    if((associationlist->findItem(result))>=0){
      FXMessageBox::question(this,MBOX_OK,"Duplicate Association","The extension given %s already exists.",result.text());
      return 1;
      }

    // Save Old
    saveFileAssociation();

    fileassociation.extension = result;
    fileassociation.type = "";
    fileassociation.command = "";
    fileassociation.mimetype = "";
    fileassociation.bigname = "";
    fileassociation.smallname = "";

    // Save New
    saveFileAssociation();

    FXint no = associationlist->appendItem(fileassociation.extension);
    associationlist->setCurrentItem(no,TRUE);
    associationlist->sortItems();
    associationlist->makeItemVisible(associationlist->getCurrentItem());
    }
  return 1;
  }


long MimeTypeDialog::onCmdDeleteFileAssociation(FXObject*,FXSelector,void*){
  if(fileassociation.extension.empty()) return 1;
  if(FXMessageBox::question(this,MBOX_OK_CANCEL,"Delete Fileassociation?","Are you sure you want to delete\nthe fileassociation for %s",fileassociation.extension.text())==MBOX_CLICKED_OK){
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension="";
    associationlist->removeItem(associationlist->getCurrentItem(),TRUE);
    }
  return 1;
  }


long MimeTypeDialog::onCmdRenameFileAssociation(FXObject*,FXSelector,void*){
  FXint no;
  FXString result=fileassociation.extension;
  if(FXInputDialog::getString(result,this,"Rename File Association","Rename File Association",NULL)){
    if(result==fileassociation.extension) return 1;
    if((no=associationlist->findItem(result))>=0){
      if(FXMessageBox::question(this,MBOX_YES_NO,"Replace Association?","Would you like to replace the existing association?")!=MBOX_CLICKED_YES){
        return 1;
        }
      }

    // Find Extension to rename
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension=result;
    // Save renamed Extension 
    saveFileAssociation();
    if(no>=0){
      FXint olditem = associationlist->getCurrentItem();
      associationlist->setCurrentItem(no,TRUE);
      // Remove old Extension
      associationlist->removeItem(olditem);
      }
    else {
      associationlist->setItemText(associationlist->getCurrentItem(),fileassociation.extension);
      }
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectIcon(FXObject*,FXSelector sel,void*){
  FXString name;
    switch(FXSELID(sel)){
	case ID_SELECT_ICONBIG_NAME: name=fileassociation.bigname; break;
	case ID_SELECT_ICONSMALL_NAME: name=fileassociation.smallname; break;
    }

    if(!FXStat::exists(name)){ name=FXPath::search(iconpath,name); }

    if(name.empty()){
      if(!iconpath.empty()){ name=iconpath.section(PATHLISTSEP,0)+PATHSEPSTRING; }
      else{ name=FXSystem::getCurrentDirectory(); }
    }

  FXFileDialog opendialog(this,"Select the Icon");
  opendialog.setSelectMode(SELECTFILE_EXISTING);
  opendialog.setFilename(name);
  opendialog.showImages(TRUE);
  if(opendialog.execute()){

	FXString iconroot=opendialog.getFilename();
	//FXString iconname=FXPath::name(iconroot);

	switch(FXSELID(sel)){
		case ID_SELECT_ICONBIG_NAME:
			updateIconApp(getApp(),button_bigname,iconroot);
			//fileassociation.bigname=iconame;
			fileassociation.bigname=iconroot;
		break;
		case ID_SELECT_ICONSMALL_NAME:
			updateIconApp(getApp(),button_smallname,iconroot);
			//fileassociation.smallname=iconname;
			fileassociation.smallname=iconroot;
		break;
	}
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectFont(FXObject*,FXSelector,void*){
  FXFontDialog fontdlg(this,"Change Body Font",DECOR_BORDER|DECOR_TITLE);
  FXFontDesc fontdesc=fontbutton->getFont()->getFontDesc();
  fontdlg.setFontDesc(fontdesc);
  if(fontdlg.execute()){
    FXFont *oldfont=bodyfont;
    fontdesc=fontdlg.getFontDesc();
    bodyfont=new FXFont(getApp(),fontdesc);
    bodyfont->create();
    fontbutton->setFont(bodyfont);
    delete oldfont;
    }
  return 1;
  }

long MimeTypeDialog::onCmdSelectFont2(FXObject*,FXSelector,void*){
  FXFontDialog fontdlg2(this,"Change Menu Font",DECOR_BORDER|DECOR_TITLE);
  FXFontDesc fontdesc2=fontbutton2->getFont()->getFontDesc();
  fontdlg2.setFontDesc(fontdesc2);
  if(fontdlg2.execute()){
    FXFont *oldfont2=menufont;
    fontdesc2=fontdlg2.getFontDesc();
    menufont=new FXFont(getApp(),fontdesc2);
    menufont->create();
    fontbutton2->setFont(menufont);
    delete oldfont2;
    }
  return 1;
  }


// Close the dialog
long MimeTypeDialog::onCmdCloseDialog(FXObject*,FXSelector,void*){

  FXint result=FXMessageBox::question(this,MBOX_SAVE_CANCEL_DONTSAVE,"Save Changes?","Do you want to save changes to the FXMovieManager Registry?\n\nThen you must restart the application for the changes to take effect.");
  if(result==MBOX_CLICKED_CANCEL){ 
	return 1;
  }
  if(result==MBOX_CLICKED_SAVE){

	saveFileAssociation();
	saveThemeColors();
	// Save Settings
  	getApp()->reg().writeIntEntry("SETTINGS","fxmmwheellines",fxmmwheellines);
	getApp()->reg().writeStringEntry("SETTINGS","mimeiconpath",iconpath.text());
  	getApp()->reg().writeStringEntry("SETTINGS","theme",theme.text());
  	getApp()->reg().writeStringEntry("SETTINGS","language",language.text());
  	getApp()->reg().writeStringEntry("SETTINGS","terminal",terminal.text());
    fontspec=fontbutton->getFont()->getFont();
	getApp()->reg().writeStringEntry("SETTINGS","bodyfont",fontspec.text());
    fontspec2=fontbutton2->getFont()->getFont();
	getApp()->reg().writeStringEntry("SETTINGS","menufont",fontspec2.text());

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
	// Thumbnailer
  	getApp()->reg().writeIntEntry("THUMBNAILER","thumbnailSize",thumbnailSize);
  	getApp()->reg().writeIntEntry("THUMBNAILER","imageQuality",imageQuality);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","filmStripOverlay",filmStripOverlay);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","workaroundIssues",workaroundIssues);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","maintainAspectRatio",maintainAspectRatio);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","smartFrameSelection",smartFrameSelection);
#endif
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailShow",autoThumbnailShow);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailCreate",autoThumbnailCreate);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailUpdate",autoThumbnailUpdate);

 	getApp()->reg().writeIntEntry("THUMBNAILER","imagethumbnailSize",imagethumbnailSize);
  }
  FXWindow::hide();
  return 1;
  }


// Restore default settings and close the dialog
long MimeTypeDialog::onCmdRestoreDefaults(FXObject*,FXSelector,void*){

  FXint result=FXMessageBox::question(this,MBOX_SAVE_CANCEL_DONTSAVE,"Restore Default Settings?","Do you want to restore the defaults to the FXMovieManager Registry?\n\nThen you must restart the application for the changes to take effect.");
  if(result==MBOX_CLICKED_CANCEL){ 
	return 1;
  }
  if(result==MBOX_CLICKED_SAVE){

	// Save File Association
	getApp()->reg().writeStringEntry("FILETYPES","dv",";videoclip;/usr/local/share/pixmaps/mime_dv.gif;/usr/local/share/pixmaps/mime_dv_small.gif;video/dv");
	getApp()->reg().writeStringEntry("FILETYPES","dif",";videoclip;/usr/local/share/pixmaps/mime_dv.gif;/usr/local/share/pixmaps/mime_dv_small.gif;video/dv");
	getApp()->reg().writeStringEntry("FILETYPES","3gp",";videoclip;/usr/local/share/pixmaps/mime_3gp.gif;/usr/local/share/pixmaps/mime_3gp_small.gif;video/3gp");
	getApp()->reg().writeStringEntry("FILETYPES","3gp2",";videoclip;/usr/local/share/pixmaps/mime_3gp.gif;/usr/local/share/pixmaps/mime_3gp_small.gif;video/3gp");
	getApp()->reg().writeStringEntry("FILETYPES","mov",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","qt",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","mp4",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","avi",";videoclip;/usr/local/share/pixmaps/mime_avi.gif;/usr/local/share/pixmaps/mime_avi_small.gif;video/x-msvideo");
	getApp()->reg().writeStringEntry("FILETYPES","flv",";videoclip;/usr/local/share/pixmaps/mime_flv.gif;/usr/local/share/pixmaps/mime_flv_small.gif;video/x-flv");
	getApp()->reg().writeStringEntry("FILETYPES","mpg",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mpe",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mpeg",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","gl",";videoclip;/usr/local/share/pixmaps/mime_gl.gif;/usr/local/share/pixmaps/mime_gl_small.gif;video/gl");
	getApp()->reg().writeStringEntry("FILETYPES","ogv",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","ogm",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","ogg",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","vob",";videoclip;/usr/local/share/pixmaps/mime_vob.gif;/usr/local/share/pixmaps/mime_vob_small.gif;video/vob");
	getApp()->reg().writeStringEntry("FILETYPES","wmv",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-wmv");
	getApp()->reg().writeStringEntry("FILETYPES","wmx",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-wmx");
	getApp()->reg().writeStringEntry("FILETYPES","asf",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-asf");
	getApp()->reg().writeStringEntry("FILETYPES","asx",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-asf");

	getApp()->reg().writeStringEntry("FILETYPES","mp2",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mp3",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mpga",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mpega",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","mid",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio2_small.gif;audio/midi");
	getApp()->reg().writeStringEntry("FILETYPES","midi",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio2_small.gif;audio/midi");
	getApp()->reg().writeStringEntry("FILETYPES","mod",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/mod");
	getApp()->reg().writeStringEntry("FILETYPES","wav",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-wav");
	getApp()->reg().writeStringEntry("FILETYPES","wma",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-ms-wma");
	getApp()->reg().writeStringEntry("FILETYPES","wax",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-ms-wax");
	getApp()->reg().writeStringEntry("FILETYPES","au",";audio;/usr/local/share/pixmaps/mime_audio4.gif;/usr/local/share/pixmaps/mime_audio4_small.gif;audio/basic");
	getApp()->reg().writeStringEntry("FILETYPES","snd",";audio;/usr/local/share/pixmaps/mime_audio4.gif;/usr/local/share/pixmaps/mime_audio4_small.gif;audio/basic");
	getApp()->reg().writeStringEntry("FILETYPES","ra",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");
	getApp()->reg().writeStringEntry("FILETYPES","ram",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");
	getApp()->reg().writeStringEntry("FILETYPES","rm",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");

	getApp()->reg().writeStringEntry("FILETYPES","bm",";image;/usr/local/share/pixmaps/mime_bmp.gif;/usr/local/share/pixmaps/mime_bmp_small.gif;image/bmp");
	getApp()->reg().writeStringEntry("FILETYPES","bmp",";image;/usr/local/share/pixmaps/mime_bmp.gif;/usr/local/share/pixmaps/mime_bmp_small.gif;image/x-windows-bmp");
	getApp()->reg().writeStringEntry("FILETYPES","png",";image;/usr/local/share/pixmaps/mime_png.gif;/usr/local/share/pixmaps/mime_png_small.gif;image/png");
	getApp()->reg().writeStringEntry("FILETYPES","jpg",";image;/usr/local/share/pixmaps/mime_jpg.gif;/usr/local/share/pixmaps/mime_jpg_small.gif;image/jpeg");
	getApp()->reg().writeStringEntry("FILETYPES","jpeg",";image;/usr/local/share/pixmaps/mime_jpg.gif;/usr/local/share/pixmaps/mime_jpg_small.gif;image/jpeg");
	getApp()->reg().writeStringEntry("FILETYPES","gif",";image;/usr/local/share/pixmaps/mime_gif.gif;/usr/local/share/pixmaps/mime_gif_small.gif;image/gif");
	getApp()->reg().writeStringEntry("FILETYPES","xpm",";image;/usr/local/share/pixmaps/mime_xpm.gif;/usr/local/share/pixmaps/mime_xpm_small.gif;image/x-xpixmap");
	getApp()->reg().writeStringEntry("FILETYPES","tif",";image;/usr/local/share/pixmaps/mime_tiff.gif;/usr/local/share/pixmaps/mime_tiff_small.gif;image/tif");
	getApp()->reg().writeStringEntry("FILETYPES","tiff",";image;/usr/local/share/pixmaps/mime_tiff.gif;/usr/local/share/pixmaps/mime_tiff_small.gif;image/tiff");

	getApp()->reg().writeStringEntry("FILETYPES","xz",";archive;/usr/local/share/pixmaps/mime_archive1.gif;/usr/local/share/pixmaps/mime_archive1_small.gif;application/x-xzip");
	getApp()->reg().writeStringEntry("FILETYPES","tar",";archive;/usr/local/share/pixmaps/mime_archive2.gif;/usr/local/share/pixmaps/mime_archive2_small.gif;application/x-tar");
	getApp()->reg().writeStringEntry("FILETYPES","tgz",";archive;/usr/local/share/pixmaps/mime_archive3.gif;/usr/local/share/pixmaps/mime_archive3_small.gif;application/x-compressed");
	getApp()->reg().writeStringEntry("FILETYPES","gz",";archive;/usr/local/share/pixmaps/mime_archive3.gif;/usr/local/share/pixmaps/mime_archive3_small.gif;application/x-gzip");
	getApp()->reg().writeStringEntry("FILETYPES","bz2",";archive;/usr/local/share/pixmaps/mime_archive4.gif;/usr/local/share/pixmaps/mime_archive4_small.gif;application/x-bzip2");
	getApp()->reg().writeStringEntry("FILETYPES","zip",";archive;/usr/local/share/pixmaps/mime_archive5.gif;/usr/local/share/pixmaps/mime_archive5_small.gif;application/x-zip-compressed");

	getApp()->reg().writeStringEntry("FILETYPES","txt",";text;/usr/local/share/pixmaps/mime_text.gif;/usr/local/share/pixmaps/mime_text_small.gif;text/text");
	getApp()->reg().writeStringEntry("FILETYPES","htm",";htm;/usr/local/share/pixmaps/mime_world1.gif;/usr/local/share/pixmaps/mime_world1_small.gif;text/htm");
	getApp()->reg().writeStringEntry("FILETYPES","html",";html;/usr/local/share/pixmaps/mime_world1.gif;/usr/local/share/pixmaps/mime_world1_small.gif;text/html");
	getApp()->reg().writeStringEntry("FILETYPES","h",";header;/usr/local/share/pixmaps/mime_header.gif;/usr/local/share/pixmaps/mime_header_small.gif;text/x-h");
	getApp()->reg().writeStringEntry("FILETYPES","hh",";header;/usr/local/share/pixmaps/mime_header.gif;/usr/local/share/pixmaps/mime_header_small.gif;text/x-h");
	getApp()->reg().writeStringEntry("FILETYPES","c",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","cc",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","cxx",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","cpp",";source;/usr/local/share/pixmaps/mime_sourceplus.gif;/usr/local/share/pixmaps/mime_sourceplus_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","c++",";source;/usr/local/share/pixmaps/mime_sourceplus.gif;/usr/local/share/pixmaps/mime_sourceplus_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","in",";configure;/usr/local/share/pixmaps/mime_configure_in.gif;/usr/local/share/pixmaps/mime_configure_in_small.gif;text/in");
	getApp()->reg().writeStringEntry("FILETYPES","ac",";configure;/usr/local/share/pixmaps/mime_configure_ac.gif;/usr/local/share/pixmaps/mime_configure_ac_small.gif;text/ac");
	getApp()->reg().writeStringEntry("FILETYPES","am",";configure;/usr/local/share/pixmaps/mime_configure_am.gif;/usr/local/share/pixmaps/mime_configure_am_small.gif;text/am");
	getApp()->reg().writeStringEntry("FILETYPES","php",";configure;/usr/local/share/pixmaps/mime_php.gif;/usr/local/share/pixmaps/mime_php_small.gif;text/x-php");
	getApp()->reg().writeStringEntry("FILETYPES","pl",";configure;/usr/local/share/pixmaps/mime_perl.gif;/usr/local/share/pixmaps/mime_perl_small.gif;text/x-perl");
	getApp()->reg().writeStringEntry("FILETYPES","pm",";configure;/usr/local/share/pixmaps/mime_perl.gif;/usr/local/share/pixmaps/mime_perl_small.gif;text/x-perl");
	getApp()->reg().writeStringEntry("FILETYPES","py",";configure;/usr/local/share/pixmaps/mime_python.gif;/usr/local/share/pixmaps/mime_python_small.gif;text/x-python");

	getApp()->reg().writeStringEntry("FILETYPES","sxw",";word;/usr/local/share/pixmaps/mime_world2.gif;/usr/local/share/pixmaps/mime_world2_small.gif;application/sword");
	getApp()->reg().writeStringEntry("FILETYPES","rtf",";word;/usr/local/share/pixmaps/mime_world5.gif;/usr/local/share/pixmaps/mime_world5_small.gif;application/rtf");
	getApp()->reg().writeStringEntry("FILETYPES","doc",";word;/usr/local/share/pixmaps/mime_world3.gif;/usr/local/share/pixmaps/mime_world3_small.gif;application/msword");
	getApp()->reg().writeStringEntry("FILETYPES","odt",";word;/usr/local/share/pixmaps/mime_world4.gif;/usr/local/share/pixmaps/mime_world4_small.gif;application/ooword");
	getApp()->reg().writeStringEntry("FILETYPES","odp",";presentation;/usr/local/share/pixmaps/mime_graph4.gif;/usr/local/share/pixmaps/mime_graph4_small.gif;application/ooimpress");
	getApp()->reg().writeStringEntry("FILETYPES","ppt",";presentation;/usr/local/share/pixmaps/mime_graph3.gif;/usr/local/share/pixmaps/mime_graph3_small.gif;application/mspowerpoint ");
	getApp()->reg().writeStringEntry("FILETYPES","ods",";spreadsheet;/usr/local/share/pixmaps/mime_calc4.gif;/usr/local/share/pixmaps/mime_calc4_small.gif;application/oocalc");
	getApp()->reg().writeStringEntry("FILETYPES","xls",";spreadsheet;/usr/local/share/pixmaps/mime_calc3.gif;/usr/local/share/pixmaps/mime_calc3_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","cvs",";spreadsheet;/usr/local/share/pixmaps/mime_calc2.gif;/usr/local/share/pixmaps/mime_calc2_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","csv",";spreadsheet;/usr/local/share/pixmaps/mime_calc2.gif;/usr/local/share/pixmaps/mime_calc2_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","odg",";drawing;/usr/local/share/pixmaps/mime_graph1.gif;/usr/local/share/pixmaps/mime_graph1_small.gif;application/oodraw");
	getApp()->reg().writeStringEntry("FILETYPES","odf",";formula;/usr/local/share/pixmaps/mime_calc1.gif;/usr/local/share/pixmaps/mime_calc1_small.gif;application/oomath");
	getApp()->reg().writeStringEntry("FILETYPES","odb",";database;/usr/local/share/pixmaps/mime_data4.gif;/usr/local/share/pixmaps/mime_data4_small.gif;application/oobase");

	getApp()->reg().writeStringEntry("FILETYPES","pls",";playlist;/usr/local/share/pixmaps/mime_list.gif;/usr/local/share/pixmaps/mime_list_small.gif;data/playlist");
	getApp()->reg().writeStringEntry("FILETYPES","pdf",";pdf;/usr/local/share/pixmaps/mime_pdf.gif;/usr/local/share/pixmaps/mime_pdf_small.gif;application/pdf");
	getApp()->reg().writeStringEntry("FILETYPES","ps",";postscript;/usr/local/share/pixmaps/mime_ps.gif;/usr/local/share/pixmaps/mime_ps_small.gif;application/pdf");

	getApp()->reg().writeStringEntry("FILETYPES","iso",";iso;/usr/local/share/pixmaps/mime_iso.gif;/usr/local/share/pixmaps/mime_iso_small.gif;file/iso");

	// UPPERCASE
	getApp()->reg().writeStringEntry("FILETYPES","DV",";videoclip;/usr/local/share/pixmaps/mime_dv.gif;/usr/local/share/pixmaps/mime_dv_small.gif;video/dv");
	getApp()->reg().writeStringEntry("FILETYPES","DIF",";videoclip;/usr/local/share/pixmaps/mime_dv.gif;/usr/local/share/pixmaps/mime_dv_small.gif;video/dv");
	getApp()->reg().writeStringEntry("FILETYPES","3GP",";videoclip;/usr/local/share/pixmaps/mime_3gp.gif;/usr/local/share/pixmaps/mime_3gp_small.gif;video/3gp");
	getApp()->reg().writeStringEntry("FILETYPES","3GP2",";videoclip;/usr/local/share/pixmaps/mime_3gp.gif;/usr/local/share/pixmaps/mime_3gp_small.gif;video/3gp");
	getApp()->reg().writeStringEntry("FILETYPES","MOV",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","QT",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","MP4",";videoclip;/usr/local/share/pixmaps/mime_mov.gif;/usr/local/share/pixmaps/mime_mov_small.gif;video/quicktime");
	getApp()->reg().writeStringEntry("FILETYPES","AVI",";videoclip;/usr/local/share/pixmaps/mime_avi.gif;/usr/local/share/pixmaps/mime_avi_small.gif;video/x-msvideo");
	getApp()->reg().writeStringEntry("FILETYPES","FLV",";videoclip;/usr/local/share/pixmaps/mime_flv.gif;/usr/local/share/pixmaps/mime_flv_small.gif;video/x-flv");
	getApp()->reg().writeStringEntry("FILETYPES","MPG",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MPE",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MPEG",";videoclip;/usr/local/share/pixmaps/mime_mpg.gif;/usr/local/share/pixmaps/mime_mpg_small.gif;video/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","GL",";videoclip;/usr/local/share/pixmaps/mime_gl.gif;/usr/local/share/pixmaps/mime_gl_small.gif;video/gl");
	getApp()->reg().writeStringEntry("FILETYPES","OGV",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","OGM",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","OGG",";videoclip;/usr/local/share/pixmaps/mime_ogg.gif;/usr/local/share/pixmaps/mime_ogg_small.gif;video/ogg");
	getApp()->reg().writeStringEntry("FILETYPES","VOB",";videoclip;/usr/local/share/pixmaps/mime_vob.gif;/usr/local/share/pixmaps/mime_vob_small.gif;video/vob");
	getApp()->reg().writeStringEntry("FILETYPES","WMV",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-wmv");
	getApp()->reg().writeStringEntry("FILETYPES","WMX",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-wmx");
	getApp()->reg().writeStringEntry("FILETYPES","ASF",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-asf");
	getApp()->reg().writeStringEntry("FILETYPES","ASX",";videoclip;/usr/local/share/pixmaps/mime_wmv.gif;/usr/local/share/pixmaps/mime_wmv_small.gif;video/x-ms-asf");

	getApp()->reg().writeStringEntry("FILETYPES","MP2",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MP3",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MPGA",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MPEGA",";audio;/usr/local/share/pixmaps/mime_audio1.gif;/usr/local/share/pixmaps/mime_audio1_small.gif;audio/mpeg");
	getApp()->reg().writeStringEntry("FILETYPES","MID",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio2_small.gif;audio/midi");
	getApp()->reg().writeStringEntry("FILETYPES","MIDI",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio2_small.gif;audio/midi");
	getApp()->reg().writeStringEntry("FILETYPES","MOD",";audio;/usr/local/share/pixmaps/mime_audio2.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/mod");
	getApp()->reg().writeStringEntry("FILETYPES","WAV",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-wav");
	getApp()->reg().writeStringEntry("FILETYPES","WMA",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-ms-wma");
	getApp()->reg().writeStringEntry("FILETYPES","WAX",";audio;/usr/local/share/pixmaps/mime_audio3.gif;/usr/local/share/pixmaps/mime_audio3_small.gif;audio/x-ms-wax");
	getApp()->reg().writeStringEntry("FILETYPES","AU",";audio;/usr/local/share/pixmaps/mime_audio4.gif;/usr/local/share/pixmaps/mime_audio4_small.gif;audio/basic");
	getApp()->reg().writeStringEntry("FILETYPES","SND",";audio;/usr/local/share/pixmaps/mime_audio4.gif;/usr/local/share/pixmaps/mime_audio4_small.gif;audio/basic");
	getApp()->reg().writeStringEntry("FILETYPES","RA",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");
	getApp()->reg().writeStringEntry("FILETYPES","RAM",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");
	getApp()->reg().writeStringEntry("FILETYPES","RM",";audio;/usr/local/share/pixmaps/mime_audio5.gif;/usr/local/share/pixmaps/mime_audio5_small.gif;audio/x-realaudio");

	getApp()->reg().writeStringEntry("FILETYPES","BM",";image;/usr/local/share/pixmaps/mime_bmp.gif;/usr/local/share/pixmaps/mime_bmp_small.gif;image/bm");
	getApp()->reg().writeStringEntry("FILETYPES","BMP",";image;/usr/local/share/pixmaps/mime_bmp.gif;/usr/local/share/pixmaps/mime_bmp_small.gif;image/x-windows-bmp");
	getApp()->reg().writeStringEntry("FILETYPES","PNG",";image;/usr/local/share/pixmaps/mime_png.gif;/usr/local/share/pixmaps/mime_png_small.gif;image/png");
	getApp()->reg().writeStringEntry("FILETYPES","JPG",";image;/usr/local/share/pixmaps/mime_jpg.gif;/usr/local/share/pixmaps/mime_jpg_small.gif;image/jpeg");
	getApp()->reg().writeStringEntry("FILETYPES","JPEG",";image;/usr/local/share/pixmaps/mime_jpg.gif;/usr/local/share/pixmaps/mime_jpg_small.gif;image/jpeg");
	getApp()->reg().writeStringEntry("FILETYPES","GIF",";image;/usr/local/share/pixmaps/mime_gif.gif;/usr/local/share/pixmaps/mime_gif_small.gif;image/gif");
	getApp()->reg().writeStringEntry("FILETYPES","XPM",";image;/usr/local/share/pixmaps/mime_xpm.gif;/usr/local/share/pixmaps/mime_xpm_small.gif;image/xpm");
	getApp()->reg().writeStringEntry("FILETYPES","TIF",";image;/usr/local/share/pixmaps/mime_tiff.gif;/usr/local/share/pixmaps/mime_tiff_small.gif;image/tif");
	getApp()->reg().writeStringEntry("FILETYPES","TIFF",";image;/usr/local/share/pixmaps/mime_tiff.gif;/usr/local/share/pixmaps/mime_tiff_small.gif;image/tiff");

	getApp()->reg().writeStringEntry("FILETYPES","XZ",";archive;/usr/local/share/pixmaps/mime_archive1.gif;/usr/local/share/pixmaps/mime_archive1_small.gif;application/x-xzip");
	getApp()->reg().writeStringEntry("FILETYPES","TAR",";archive;/usr/local/share/pixmaps/mime_archive2.gif;/usr/local/share/pixmaps/mime_archive2_small.gif;application/x-tar");
	getApp()->reg().writeStringEntry("FILETYPES","TGZ",";archive;/usr/local/share/pixmaps/mime_archive3.gif;/usr/local/share/pixmaps/mime_archive3_small.gif;application/x-compressed");
	getApp()->reg().writeStringEntry("FILETYPES","GZ",";archive;/usr/local/share/pixmaps/mime_archive3.gif;/usr/local/share/pixmaps/mime_archive3_small.gif;application/x-gzip");
	getApp()->reg().writeStringEntry("FILETYPES","BZ2",";archive;/usr/local/share/pixmaps/mime_archive4.gif;/usr/local/share/pixmaps/mime_archive4_small.gif;application/x-bzip2");
	getApp()->reg().writeStringEntry("FILETYPES","ZIP",";archive;/usr/local/share/pixmaps/mime_archive5.gif;/usr/local/share/pixmaps/mime_archive5_small.gif;application/x-zip-compressed");

	getApp()->reg().writeStringEntry("FILETYPES","TXT",";text;/usr/local/share/pixmaps/mime_text.gif;/usr/local/share/pixmaps/mime_text_small.gif;text/text");
	getApp()->reg().writeStringEntry("FILETYPES","HTM",";htm;/usr/local/share/pixmaps/mime_world1.gif;/usr/local/share/pixmaps/mime_world1_small.gif;text/htm");
	getApp()->reg().writeStringEntry("FILETYPES","HTML",";html;/usr/local/share/pixmaps/mime_world1.gif;/usr/local/share/pixmaps/mime_world1_small.gif;text/html");
	getApp()->reg().writeStringEntry("FILETYPES","H",";header;/usr/local/share/pixmaps/mime_header.gif;/usr/local/share/pixmaps/mime_header_small.gif;text/x-h");
	getApp()->reg().writeStringEntry("FILETYPES","HH",";header;/usr/local/share/pixmaps/mime_header.gif;/usr/local/share/pixmaps/mime_header_small.gif;text/x-h");
	getApp()->reg().writeStringEntry("FILETYPES","C",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","CC",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","CXX",";source;/usr/local/share/pixmaps/mime_source.gif;/usr/local/share/pixmaps/mime_source_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","CPP",";source;/usr/local/share/pixmaps/mime_sourceplus.gif;/usr/local/share/pixmaps/mime_sourceplus_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","C++",";source;/usr/local/share/pixmaps/mime_sourceplus.gif;/usr/local/share/pixmaps/mime_sourceplus_small.gif;text/x-c");
	getApp()->reg().writeStringEntry("FILETYPES","IN",";configure;/usr/local/share/pixmaps/mime_configure_in.gif;/usr/local/share/pixmaps/mime_configure_in_small.gif;text/in");
	getApp()->reg().writeStringEntry("FILETYPES","AC",";configure;/usr/local/share/pixmaps/mime_configure_ac.gif;/usr/local/share/pixmaps/mime_configure_ac_small.gif;text/ac");
	getApp()->reg().writeStringEntry("FILETYPES","AM",";configure;/usr/local/share/pixmaps/mime_configure_am.gif;/usr/local/share/pixmaps/mime_configure_am_small.gif;text/am");
	getApp()->reg().writeStringEntry("FILETYPES","PHP",";configure;/usr/local/share/pixmaps/mime_php.gif;/usr/local/share/pixmaps/mime_php_small.gif;text/x-php");
	getApp()->reg().writeStringEntry("FILETYPES","PL",";configure;/usr/local/share/pixmaps/mime_perl.gif;/usr/local/share/pixmaps/mime_perl_small.gif;text/x-perl");
	getApp()->reg().writeStringEntry("FILETYPES","PM",";configure;/usr/local/share/pixmaps/mime_perl.gif;/usr/local/share/pixmaps/mime_perl_small.gif;text/x-perl");
	getApp()->reg().writeStringEntry("FILETYPES","PY",";configure;/usr/local/share/pixmaps/mime_python.gif;/usr/local/share/pixmaps/mime_python_small.gif;text/x-python");

	getApp()->reg().writeStringEntry("FILETYPES","SXW",";word;/usr/local/share/pixmaps/mime_world2.gif;/usr/local/share/pixmaps/mime_world2_small.gif;application/sword");
	getApp()->reg().writeStringEntry("FILETYPES","RTF",";word;/usr/local/share/pixmaps/mime_world5.gif;/usr/local/share/pixmaps/mime_world5_small.gif;application/rtf");
	getApp()->reg().writeStringEntry("FILETYPES","DOC",";word;/usr/local/share/pixmaps/mime_world3.gif;/usr/local/share/pixmaps/mime_world3_small.gif;application/msword");
	getApp()->reg().writeStringEntry("FILETYPES","ODT",";word;/usr/local/share/pixmaps/mime_world4.gif;/usr/local/share/pixmaps/mime_world4_small.gif;application/ooword");
	getApp()->reg().writeStringEntry("FILETYPES","ODP",";presentation;/usr/local/share/pixmaps/mime_graph4.gif;/usr/local/share/pixmaps/mime_graph4_small.gif;application/ooimpress");
	getApp()->reg().writeStringEntry("FILETYPES","PPT",";presentation;/usr/local/share/pixmaps/mime_graph3.gif;/usr/local/share/pixmaps/mime_graph3_small.gif;application/mspowerpoint ");
	getApp()->reg().writeStringEntry("FILETYPES","ODS",";spreadsheet;/usr/local/share/pixmaps/mime_calc4.gif;/usr/local/share/pixmaps/mime_calc4_small.gif;application/oocalc");
	getApp()->reg().writeStringEntry("FILETYPES","XLS",";spreadsheet;/usr/local/share/pixmaps/mime_calc3.gif;/usr/local/share/pixmaps/mime_calc3_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","CVS",";spreadsheet;/usr/local/share/pixmaps/mime_calc2.gif;/usr/local/share/pixmaps/mime_calc2_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","CSV",";spreadsheet;/usr/local/share/pixmaps/mime_calc2.gif;/usr/local/share/pixmaps/mime_calc2_small.gif;application/msexcel ");
	getApp()->reg().writeStringEntry("FILETYPES","ODG",";drawing;/usr/local/share/pixmaps/mime_graph1.gif;/usr/local/share/pixmaps/mime_graph1_small.gif;application/oodraw");
	getApp()->reg().writeStringEntry("FILETYPES","ODF",";formula;/usr/local/share/pixmaps/mime_calc1.gif;/usr/local/share/pixmaps/mime_calc1_small.gif;application/oomath");
	getApp()->reg().writeStringEntry("FILETYPES","ODB",";database;/usr/local/share/pixmaps/mime_data4.gif;/usr/local/share/pixmaps/mime_data4_small.gif;application/oobase");

	getApp()->reg().writeStringEntry("FILETYPES","PLS",";playlist;/usr/local/share/pixmaps/mime_list.gif;/usr/local/share/pixmaps/mime_list_small.gif;data/playlist");
	getApp()->reg().writeStringEntry("FILETYPES","PDF",";pdf;/usr/local/share/pixmaps/mime_pdf.gif;/usr/local/share/pixmaps/mime_pdf_small.gif;application/pdf");
	getApp()->reg().writeStringEntry("FILETYPES","PS",";postscript;/usr/local/share/pixmaps/mime_ps.gif;/usr/local/share/pixmaps/mime_ps_small.gif;application/pdf");

	getApp()->reg().writeStringEntry("FILETYPES","ISO",";iso;/usr/local/share/pixmaps/mime_iso.gif;/usr/local/share/pixmaps/mime_iso_small.gif;file/iso");

	language="en";

	theme="dark";
	theme_fxmmfore   = FXRGB(200,200,200);
	theme_fxmmbase   = FXRGB( 60, 60, 60);
	theme_fxmmborder = FXRGB(  0,  0,  0);
	theme_fxmmback   = FXRGB( 20, 20, 20);
	theme_fxmmmenufore  = FXRGB(200,200,200);
	theme_fxmmmenuback  = FXRGB( 10, 10, 10);
	theme_fxmmselfore   = FXRGB(  0,  0,  0);
	theme_fxmmselback   = FXRGB(100,100,100);
	theme_fxmmtipfore  = FXRGB(200,200,200);
	theme_fxmmtipback  = FXRGB( 20, 20, 20);
	theme_fxmmhilite  = FXRGB(100,100,100);
	theme_fxmmshadow  = FXRGB( 10, 10, 10);
	saveThemeColors();

	// Save Settings
  	getApp()->reg().writeIntEntry("SETTINGS","fxmmwheellines",4);
	getApp()->reg().writeStringEntry("SETTINGS","mimeiconpath","/usr/local/share/pixmaps");
  	getApp()->reg().writeStringEntry("SETTINGS","theme",theme.text());
  	getApp()->reg().writeStringEntry("SETTINGS","language",language.text());
  	getApp()->reg().writeStringEntry("SETTINGS","terminal","/usr/bin/uxterm");
	getApp()->reg().writeStringEntry("SETTINGS","bodyfont","Fixed [Misc],100,normal,regular");
	getApp()->reg().writeStringEntry("SETTINGS","menufont","Fixed [Misc],100,normal,regular");

#ifdef HAVE_LIBFFMPEGTHUMBNAILER
	// Thumbnailer
  	getApp()->reg().writeIntEntry("THUMBNAILER","thumbnailSize",128);
  	getApp()->reg().writeIntEntry("THUMBNAILER","imageQuality",8);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","filmStripOverlay",false);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","workaroundIssues",false);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","maintainAspectRatio",true);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","smartFrameSelection",false);
#endif
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailShow",true);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailCreate",false);
  	getApp()->reg().writeBoolEntry("THUMBNAILER","autoThumbnailUpdate",false);

  	getApp()->reg().writeIntEntry("THUMBNAILER","imagethumbnailSize",128);
  }

  FXWindow::hide();
  return 1;
  }

long MimeTypeDialog::onCmdCustomLanguage(FXObject*,FXSelector,void*){
    languagedialog->show(PLACEMENT_OWNER);
  return 1;
  }


// EOF
