/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2009,2010 by Luigi Ziviani <ziviani_luigi@yahoo.it>.        *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: fxarchive.cpp,   v 1.1                       2008/01/01                   *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     *
********************************************************************************/
#include <fx.h>
#include "icons.h"
#include "fxarchive.h"
#include "config.h"


// Message Map for the Archive Window class
FXDEFMAP(ArchiveDialog) ArchiveDialogMap[]={

  //________Message_Type_____________________ID____________Message_Handler_______
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_QUIT,  		ArchiveDialog::onCmdQuit),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_TAR,  ArchiveDialog::onCmdCompressTAR),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_TAR,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_GZ,  	ArchiveDialog::onCmdCompressGZ),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_GZ,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_BZ2,  ArchiveDialog::onCmdCompressBZ2),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_BZ2,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_XZ, 	ArchiveDialog::onCmdCompressXZ),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_XZ,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_ZIP,  ArchiveDialog::onCmdCompressZIP),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_ZIP,	ArchiveDialog::onUpdSelectedCompress),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMPRESS_7ZIP, ArchiveDialog::onCmdCompress7ZIP),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_COMPRESS_7ZIP,	ArchiveDialog::onUpdSelectedCompress),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_EXTRACT,   	ArchiveDialog::onCmdExtract),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_EXTRACT,		ArchiveDialog::onUpdSelectedExtract),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_COMMAND,   	ArchiveDialog::onCmdCommand),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_TERMINAL,   	ArchiveDialog::onCmdTerminal),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_OPEN,			ArchiveDialog::onCmdOpen),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_OPENWITH,		ArchiveDialog::onCmdOpenWith),

  FXMAPFUNC(SEL_DOUBLECLICKED, 	   ArchiveDialog::ID_FILELIST,		ArchiveDialog::onCmdFilelist),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,ArchiveDialog::ID_FILELIST,		ArchiveDialog::onFileListPopup),
  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_DIRLIST,		ArchiveDialog::onCmdDirlist),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_FILEDELETE,	ArchiveDialog::onCmdFileDelete),
  FXMAPFUNC(SEL_UPDATE,			   ArchiveDialog::ID_FILEDELETE,	ArchiveDialog::onUpdFileSelected),

  FXMAPFUNC(SEL_COMMAND,		   ArchiveDialog::ID_ABOUT,   	    ArchiveDialog::onCmdAbout),
  };


// Macro for the ArchiveApp class hierarchy implementation
FXIMPLEMENT(ArchiveDialog,FXMainWindow,ArchiveDialogMap,ARRAYNUMBER(ArchiveDialogMap))

// Create Archive dialog
ArchiveDialog::ArchiveDialog(FXApp *a)
	:FXMainWindow(a,"FXArchive Utility",NULL,NULL,DECOR_ALL,0,0,700,400){

  // set icon on the main window
  logoicon=new FXPNGIcon(getApp(),fxar64);
  logominiicon=new FXPNGIcon(getApp(),fxar16);
  setIcon(logoicon);
  setMiniIcon(logominiicon);

  accepticon=new FXGIFIcon(getApp(),m_accept);
  cancelicon=new FXGIFIcon(getApp(),m_cancel);
  canceliconb=new FXGIFIcon(getApp(),m_cancel_big);
  extracticon=new FXGIFIcon(getApp(),m_archivextr);
  extracticonb=new FXGIFIcon(getApp(),m_archivextr_big);
  compressicons1=new FXGIFIcon(getApp(),mime_archive1_small);
  compressicons2=new FXGIFIcon(getApp(),mime_archive2_small);
  compressicons3=new FXGIFIcon(getApp(),mime_archive3_small);
  compressicons4=new FXGIFIcon(getApp(),mime_archive4_small);
  compressicons5=new FXGIFIcon(getApp(),mime_archive5_small);
  compressicon1=new FXGIFIcon(getApp(),mime_archive1);
  compressicon2=new FXGIFIcon(getApp(),mime_archive2);
  compressicon3=new FXGIFIcon(getApp(),mime_archive3);
  compressicon4=new FXGIFIcon(getApp(),mime_archive4);
  compressicon5=new FXGIFIcon(getApp(),mime_archive5);
  m_refreshicon=new FXGIFIcon(getApp(),m_refresh);
  m_terminalicon=new FXGIFIcon(getApp(),m_terminal);
  m_moveupicon=new FXGIFIcon(getApp(),m_up);
  m_appendicon=new FXGIFIcon(getApp(),m_append);
  m_deleteicon=new FXGIFIcon(getApp(),m_delete);
  m_renamediricon=new FXGIFIcon(getApp(),m_renamedir);
  m_consoleicon=new FXGIFIcon(getApp(),m_console);
  // filelist header
  m_usricon=new FXGIFIcon(getApp(),m_usr);
  m_grpicon=new FXGIFIcon(getApp(),m_grp);
  m_prmicon=new FXGIFIcon(getApp(),m_prm);
  m_dimicon=new FXGIFIcon(getApp(),m_dim);
  m_daticon=new FXGIFIcon(getApp(),m_dat);
  m_diricon=new FXGIFIcon(getApp(),m_dir);

  if(!readConfig()){
	  printf("Configuration not loaded!");
  }
	// menubar
  menubar = new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

  dialogframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0,0,0);
  filesframe=new FXHorizontalFrame(dialogframe,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0,0,0);
  splitterh=new FXSplitter(filesframe,LAYOUT_SIDE_TOP|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_HORIZONTAL|SPLITTER_REVERSED);
  dirlist=new FXDirList(splitterh,this,ID_DIRLIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT|DIRLIST_NO_OWN_ASSOC,0,0,0,0);
  filelist=new FXFileList(splitterh,this,ID_FILELIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC,0,0,500,0);

  // Arrange menu
  menu_arrange=new FXMenuPane(this);
  title_arrange=new FXMenuTitle(menubar,"&Arrange",NULL,menu_arrange);
  radio_det=new FXMenuRadio(menu_arrange,"Detailed &List\t\tView as detailed list.",filelist,FXFileList::ID_SHOW_DETAILS);
  radio_sma=new FXMenuRadio(menu_arrange,"&Small icons\t\tView as small icons.",filelist,FXFileList::ID_SHOW_MINI_ICONS);
  radio_big=new FXMenuRadio(menu_arrange,"&Big icons\t\tView as big icons.",filelist,FXFileList::ID_SHOW_BIG_ICONS);

  // Show/Hide menu
  menu_show=new FXMenuPane(this);
  title_show=new FXMenuTitle(menubar,"&Show/Hide",NULL,menu_show);
  check_fil=new FXMenuCheck(menu_show,"Hidden &Directories\t\tShow hidden directories.",dirlist,FXDirList::ID_TOGGLE_HIDDEN);
  check_dir=new FXMenuCheck(menu_show,"Hidden &Files\t\tShow hidden files and directories.",filelist,FXFileList::ID_TOGGLE_HIDDEN);
  new FXMenuSeparator(menu_show);
  check_img=new FXMenuCheck(menu_show,"&Preview Images\t\tShow thumbnail images.",filelist,FXFileList::ID_TOGGLE_IMAGES);

  // About menu
  menu_about=new FXMenuPane(this);
  title_about=new FXMenuTitle(menubar,"&About",NULL,menu_about,LAYOUT_RIGHT);
  cmd_about=new FXMenuCommand(menu_about,"&About\t\tAbout FXArchive",NULL,this,ID_ABOUT);

  // buttons
  buttonsframe=new FXVerticalFrame(splitterh,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH,0,0,0,0, 0,0,0,0);
#ifndef DISABLE_TAR
  button_ta=new FXButton(buttonsframe,tr(" &TAR"),compressicon2,this,ID_COMPRESS_TAR,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
#ifndef DISABLE_GZIP
  button_gz=new FXButton(buttonsframe,tr(" &GZ"),compressicon3,this,ID_COMPRESS_GZ,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
#ifndef DISABLE_BZIP2
  button_bz=new FXButton(buttonsframe,tr(" &BZ2"),compressicon4,this,ID_COMPRESS_BZ2,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
#ifndef DISABLE_XZIP
  button_xz=new FXButton(buttonsframe,tr(" &XZ"),compressicon1,this,ID_COMPRESS_XZ,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
#ifndef DISABLE_ZIP
  button_zi=new FXButton(buttonsframe,tr(" &ZIP"),compressicon5,this,ID_COMPRESS_ZIP,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
#ifndef DISABLE_SEVENZIP
  button_7z=new FXButton(buttonsframe,tr(" &7ZIP"),compressicon5,this,ID_COMPRESS_7ZIP,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
#endif
  button_ex=new FXButton(buttonsframe,tr(" &Extract"),extracticonb,this,ID_EXTRACT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y);
  button_qu=new FXButton(buttonsframe,tr(" &Quit"),canceliconb,this,ID_QUIT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,20,20);
  // button to move up
  new FXButton(splitterh,"\tUp one level\tGo up to higher directory.",m_moveupicon,filelist,FXFileList::ID_DIRECTORY_UP,BUTTON_TOOLBAR|FRAME_RAISED|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_Y);

  new FXHorizontalSeparator(dialogframe,SEPARATOR_GROOVE|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);

  FXHorizontalFrame *copyright=new FXHorizontalFrame(dialogframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  FXLabel *label = new FXLabel(copyright,"FXArchive - Copyright (C) Luigi Ziviani",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();
}

// Destroy
ArchiveDialog::~ArchiveDialog(){
  delete cancelicon;
  delete accepticon;
  delete extracticon;
  delete compressicon1;
  delete compressicon2;
  delete compressicon3;
  delete compressicon4;
  delete compressicon5;
  delete compressicons1;
  delete compressicons2;
  delete compressicons3;
  delete compressicons4;
  delete compressicons5;
  delete m_refreshicon;
  delete m_terminalicon;
  delete m_moveupicon;
  delete m_appendicon;
  delete m_deleteicon;
  delete m_consoleicon;
  delete m_renamediricon;
  // filelist header
  delete m_usricon;
  delete m_grpicon;
  delete m_prmicon;
  delete m_dimicon;
  delete m_daticon;
  delete m_diricon;
  }


// Execute Compress Command
FXbool ArchiveDialog::executeCompressCommand(FXFileList* panel, const FXString& dlg_cmd, const FXString& dlg_ext, const FXString& dlg_label, FXint dlg_form){

    FXString filetocomp,file, executable;
    for(FXint i=0; i<panel->getNumItems(); i++){
      if(panel->isItemSelected(i)){
        file=panel->getItemFilename(i);
        if(file=="..") continue;
        filetocomp=FXPath::name(file);

		if (dlg_form==1)    { executable="cd "+FXPath::enquote(panel->getDirectory())+"; "+dlg_cmd+filetocomp+dlg_ext+filetocomp+" &"; }
		else if(dlg_form==2){ executable="cd "+FXPath::enquote(panel->getDirectory())+"; "+dlg_cmd+filetocomp+" &"; }

		FXDialogBox dialog(this,dlg_label,DECOR_TITLE|DECOR_BORDER);
		FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
		new FXLabel(content,filetocomp,NULL,LAYOUT_FILL_X);
  	  	FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  	  	new FXButton(buttons,"&YES",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
  	  	new FXButton(buttons,"&Cancel",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
  	  	dialog.create();

  	  	if(dialog.execute()){
	        int ret = system(executable.text());
	        if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT)){
    	        return FALSE;
	     	}
       	}
      }
    }
  return TRUE;
  }

// Compress TAR
void ArchiveDialog::CompressTAR(FXFileList* panel){
#ifndef DISABLE_TAR
    dlg_cmd="tar -cf ";
	dlg_ext =".tar ";
	dlg_label="COMPRESSING: tar -cf";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress GZ
void ArchiveDialog::CompressGZ(FXFileList* panel){
#ifndef DISABLE_GZIP
    dlg_cmd="gzip ";
	dlg_ext =".gz ";
	dlg_label="COMPRESSING: gzip";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress BZ2
void ArchiveDialog::CompressBZ2(FXFileList* panel){
#ifndef DISABLE_BZIP2
    dlg_cmd="bzip2 -z ";
	dlg_ext =".bz2 ";
	dlg_label="COMPRESSING: bzip2 -z";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress XZ
void ArchiveDialog::CompressXZ(FXFileList* panel){
#ifndef DISABLE_XZIP
    dlg_cmd="xz -z ";
	dlg_ext =".xz ";
	dlg_label="COMPRESSING: xz -z";
	dlg_form=2;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress ZIP
void ArchiveDialog::CompressZIP(FXFileList* panel){
#ifndef DISABLE_ZIP
    dlg_cmd="zip ";
	dlg_ext =".zip -xi ";
	dlg_label="COMPRESSING: zip -xi";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }

// Compress 7Z
void ArchiveDialog::Compress7ZIP(FXFileList* panel){
#ifndef DISABLE_SEVENZIP
    dlg_cmd="7z a ";
	dlg_ext =".7z ";
	dlg_label="COMPRESSING: 7z a";
	dlg_form=1;
	if(!executeCompressCommand(panel, dlg_cmd, dlg_ext, dlg_label, dlg_form)){
  	  	FXMessageBox::error(this,MBOX_OK,"COMPRESSION NOT EXECUTED","Operation Failed!");
	}
#endif
  return;
  }


// Extract archive
FXbool ArchiveDialog::extractArchive(FXString& file){
  FXString command,filename,rootname;

		// determine extension
    	filename=FXPath::absolute(filelist->getDirectory(),file);
		FXString ext=FXPath::extension(filename);
		printf("ext: %s\n", ext.text());

		// determine sub-extension
		rootname=FXPath::stripExtension(file);
		FXString root=FXPath::extension(rootname);
		printf("root: %s\n", root.text());

	  	// Determine type of archive
		if(comparecase(ext,"gz")==0){
			if(comparecase(root,"tar")==0){
				command="tar xf "+file+" --gunzip &";
			}
			else if(comparecase(root,"tar")!=0){
				command="gunzip "+file+" &";
			}
		}
		else if(comparecase(ext,"bz2")==0){
			if(comparecase(root,"tar")==0){
				command="tar xf "+file+" --bzip2 &";
			}
			else if(comparecase(root,"tar")!=0){
				command="bunzip2 "+file+" &";
			}
		}
		else if(comparecase(ext,"xz")==0){
			if(comparecase(root,"tar")==0){
				command="tar xf "+file+" --xz &";
			}
			else if(comparecase(root,"tar")!=0){
				command="xz -d "+file+" &";
			}
		}
		else if(comparecase(ext,"lzma")==0){
			if(comparecase(root,"tar")==0){
				command="tar xf "+file+" --lzma &";
			}
			else if(comparecase(root,"tar")!=0){
				command="lzma -d "+file+" &";
			}
		}
		
		else if(comparecase(ext,"tgz")==0){
			command="tar -zxf "+file+" &";
		}
		else if(comparecase(ext,"tar")==0){
			command="tar -xf "+file+" &";
		}
		else if(comparecase(ext,"zip")==0){
			command="unzip "+file+" &";
		}
		else if(comparecase(ext,"rar")==0){
	  		command="unrar x "+file+" &";
		}
		else if(comparecase(ext,"7z")==0){
	  		command="7z x "+file+" &";
		}
		else{
	  		FXMessageBox::error(this,MBOX_OK,"COMMAND FAILURE","Not suitable extension found %s\n",root.text());
	  		return FALSE;
		}

		FXString executable="cd "+FXPath::enquote(filelist->getDirectory())+"; "+command;

  		FXDialogBox dialog(this,"EXTRACTING",DECOR_TITLE|DECOR_BORDER);
		FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
  		new FXLabel(content,file,NULL,LAYOUT_FILL_X);
  		FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  		new FXButton(buttons,"&YES",accepticon,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
  		new FXButton(buttons,"&Cancel",cancelicon,&dialog,FXDialogBox::ID_CANCEL,ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
  		dialog.create();

  		if(dialog.execute()){
    		int ret = system(executable.text());
        	if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT)){
    	        return FALSE;
	    	}
    	}

  return TRUE;
  }



// Compress TAR
long ArchiveDialog::onCmdCompressTAR(FXObject*,FXSelector,void*){
    CompressTAR(filelist);
    return 1;
  }

// Compress XZ
long ArchiveDialog::onCmdCompressXZ(FXObject*,FXSelector,void*){
	CompressXZ(filelist);
  	return 1;
  }

// Compress ZIP
long ArchiveDialog::onCmdCompressZIP(FXObject*,FXSelector,void*){
	CompressZIP(filelist);
  	return 1;
  }

// Compress 7Z
long ArchiveDialog::onCmdCompress7ZIP(FXObject*,FXSelector,void*){
	Compress7ZIP(filelist);
    return 1;
  }

// Compress GZ
long ArchiveDialog::onCmdCompressGZ(FXObject*,FXSelector,void*){
	CompressGZ(filelist);
    return 1;
  }

// Compress BZ2
long ArchiveDialog::onCmdCompressBZ2(FXObject*,FXSelector,void*){
	CompressBZ2(filelist);
    return 1;
  }




long ArchiveDialog::onCmdExtract(FXObject*,FXSelector,void*){

  for(FXint i=0; i<filelist->getNumItems(); i++){
    if(filelist->isItemSelected(i)){
        FXString file=filelist->getItemFilename(i);
        if(file=="..") continue;

		// determine extension
    	filename=FXPath::absolute(filelist->getDirectory(),file);
        FXString ext=FXPath::extension(FXPath::name(filename));
        // printf("extension: %s\n",ext.text());
        if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"tar")==0){
          extractArchive(filename);
        }
    }
  }
  return 1;
}



// Popup menu for item in file list
long ArchiveDialog::onFileListPopup(FXObject*,FXSelector,void* ptr){
  FXEvent *event=(FXEvent*)ptr;
  if(event->moved) return 1;
  FXint index=filelist->getItemAt(event->win_x,event->win_y);
  FXMenuPane pane(filelist);
  FXMenuPane *compressmenu;

  // We clicked in the background
  if(index<0){
    FXLabel *filebacklabel = new FXLabel(&pane,"Background");
    filebacklabel->setTextColor(FXRGB(120,0,0));
    new FXMenuCommand(&pane,"Refresh",m_refreshicon,filelist,FXFileList::ID_REFRESH);
    new FXMenuCommand(&pane,"Terminal",m_terminalicon,this,ID_TERMINAL);
    new FXMenuCommand(&pane,"Run here",m_consoleicon,this,ID_COMMAND);
    }

  // We clicked on an item file
  else{
    FXLabel *fileforelabel = new FXLabel(&pane,"File Item");
    fileforelabel->setTextColor(FXRGB(120,0,0));

    new FXMenuCommand(&pane," Extract",extracticon,this,ID_EXTRACT);
#ifndef DISABLE_TAR
    new FXMenuCommand(&pane," Tar",compressicons2,this,ID_COMPRESS_TAR);
#endif
    // Compress menu
    compressmenu=new FXMenuPane(&pane);
#ifndef DISABLE_GZIP
    new FXMenuCommand(compressmenu," GZIP",compressicons3,this,ID_COMPRESS_GZ);
#endif
#ifndef DISABLE_BZIP2
    new FXMenuCommand(compressmenu," BZIP2",compressicons4,this,ID_COMPRESS_BZ2);
#endif
#ifndef DISABLE_XZIP
    new FXMenuCommand(compressmenu," XZIP",compressicons1,this,ID_COMPRESS_XZ);
#endif
#ifndef DISABLE_SEVENZIP
    new FXMenuCommand(compressmenu," 7ZIP",compressicons5,this,ID_COMPRESS_7ZIP);
#endif
#ifndef DISABLE_ZIP
    new FXMenuCommand(compressmenu," ZIP",compressicons5,this,ID_COMPRESS_ZIP);
#endif
    new FXMenuCascade(&pane,"Compress",compressicons1,compressmenu);

    new FXMenuSeparator(&pane);
    new FXMenuCommand(&pane,"Open",m_appendicon,this,ID_OPEN);
    new FXMenuCommand(&pane,"Open with",m_appendicon,this,ID_OPENWITH);
    new FXMenuSeparator(&pane);
    new FXMenuCommand(&pane," Delete",m_deleteicon,this,ID_FILEDELETE);
    }

  pane.create();
  pane.popup(NULL,event->root_x,event->root_y);
  getApp()->runModalWhileShown(&pane);
  return 1;
  }



// Pop up terminal
long ArchiveDialog::onCmdTerminal(FXObject*,FXSelector,void*){
  FXString executable="cd "+FXPath::enquote(filelist->getDirectory())+"; "+terminal+" &";
  system(executable.text());
  return 1;
  }



long ArchiveDialog::onCmdCommand(FXObject*,FXSelector,void*){
  FXString archcommand=getApp()->reg().readStringEntry("FXArchive","archcommand","md5sum fxmm &> fxmm.sum");

  if(FXInputDialog::getString(archcommand,this,"Run a command here","Command:")){
    getApp()->reg().writeStringEntry("FXArchive","archcommand",archcommand.text());
    archcommand="cd "+FXPath::enquote(filelist->getDirectory())+"; "+archcommand+" &";
    // printf("runcommand: %s\n", archcommand.text());
    system(archcommand.text());
    }
  return 1;
}


// Open with associated Command
long ArchiveDialog::onCmdOpen(FXObject*,FXSelector,void*){
  FXint index=filelist->getCurrentItem();
  if(index>=0){

    // If directory, open the directory
    if(filelist->isItemDirectory(index)){
      FXString directory=filelist->getItemPathname(index);
      FXTRACE((100,"directory=%s\n",directory.text()));
      filelist->setDirectory(directory);
      dirlist->setDirectory(directory);
      }

    // If regular file return as the selected file
    else if(filelist->isItemFile(index)){

      FXFileAssoc *fileassoc=filelist->getItemAssoc(index);
      if(fileassoc){
        if(fileassoc->command.text()){
          FXString command=FXString::value(fileassoc->command.text());
	      FXString runcommand="cd "+FXPath::enquote(filelist->getDirectory())+"; "+command+" "+filelist->getItemFilename(index)+" &";
          FXTRACE((100,"system(%s)\n",command.text()));
	      // printf("runcommand: %s\n", runcommand.text()); 
          system(runcommand.text());
          }
        else{
          FXMessageBox::information(this,MBOX_OK,"Unknown Command","No command has been defined for this file: %s",filelist->getItemFilename(index).text());
          }
        }
      else{
        FXMessageBox::information(this,MBOX_OK,"Unknown Mime Type","No Association has been set for this file: %s",filelist->getItemFilename(index).text());
        }
      }
  }
  return 1;
}



// Open with program
long ArchiveDialog::onCmdOpenWith(FXObject*,FXSelector,void*){
  FXString command=getApp()->reg().readStringEntry("SETTINGS","command","fxite");
  FXString filename=filelist->getCurrentFile();

  if(FXInputDialog::getString(command,this,"Open File","Open " + FXPath::name(filename) + " with:")){
    getApp()->reg().writeStringEntry("SETTINGS","command",command.text());
    command=command+" "+filename+" &";
    // printf("command: %s\n", command.text()); 
    system(command.text());
    }
  return 1;
}



// Close dialog with a cancel
long ArchiveDialog::onCmdQuit(FXObject*,FXSelector,void*){
  // Write new window size and position back to registry
#ifdef HAVE_LIBFOX_1_7
  getApp()->reg().writeUIntEntry("FXArchive","width",(FXuint)getWidth());
  getApp()->reg().writeUIntEntry("FXArchive","height",(FXuint)getHeight());
#endif
#ifdef HAVE_LIBFOX_1_6
  getApp()->reg().writeUnsignedEntry("FXArchive","width",(FXuint)getWidth());
  getApp()->reg().writeUnsignedEntry("FXArchive","height",(FXuint)getHeight());
#endif
  getApp()->reg().writeIntEntry("FXArchive","xpos",getX());
  getApp()->reg().writeIntEntry("FXArchive","ypos",getY());
  // set currentdir
  currentdir = dirlist->getDirectory();
  getApp()->reg().writeStringEntry("FXArchive","currentdir",currentdir.text());
  // Write registry settings
  getApp()->reg().write();
  // Quit
  getApp()->exit(0);
}



// Command when double clicking in the filelist
long ArchiveDialog::onCmdFilelist(FXObject*,FXSelector,void* ptr){
  FXint index=(FXint)(FXival)ptr;

  if(0<=index){
    if(filelist->isItemDirectory(index)){
      currentdir=filelist->getItemPathname(index);
      filelist->setDirectory(currentdir);
      dirlist->setDirectory(currentdir);
      }
    else if(filelist->isItemFile(index)){
      filename=filelist->getItemPathname(index);
	  FXString ext=FXPath::extension(FXPath::name(filename));
	  // printf("extension: %s\n",ext.text());
	  // autoextract if double clicking
	  if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0||comparecase(ext,"tar")==0){
		extractArchive(filename);
	  }
    }
  }
  return 1;
}



// Change the directory in the filelist when clicking in the dirlist
long ArchiveDialog::onCmdDirlist(FXObject*,FXSelector,void* ptr){
  FXTreeItem *item=(FXTreeItem*)ptr;
  FXString path=dirlist->getItemPathname(item);
  filelist->setDirectory(path);
  return 1;
  }



// Enable sender when Files have been selected
long ArchiveDialog::onUpdSelectedCompress(FXObject* sender,FXSelector,void*){
	FXString filename;
    for(FXint i=0; i<filelist->getNumItems(); i++){
		if(filelist->isItemSelected(i)){
       		filename=filelist->getItemFilename(i);
	  		FXString ext=FXPath::extension(FXPath::name(filename));
	  		// printf("extension: %s\n",ext.text());
	  		if(comparecase(ext,"gz")==0||comparecase(ext,"xz")==0||comparecase(ext,"lzma")==0||comparecase(ext,"bz2")==0||comparecase(ext,"zip")==0||comparecase(ext,"tgz")==0||comparecase(ext,"7z")==0){
				return 1;
			}
		}
	}
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
}



// Enable sender when Files have been selected
long ArchiveDialog::onUpdSelectedExtract(FXObject* sender,FXSelector,void*){
	FXString filename, ext;
    for(FXint i=0; i<filelist->getNumItems(); i++){
		if(filelist->isItemSelected(i)){
			if(filelist->isItemDirectory(i)){
				return 1;
			}
       		filename=filelist->getItemFilename(i);
	  		ext=FXPath::extension(FXPath::name(filename));
	  		//printf("extension: %s\n",ext.text());
	  		if(comparecase(ext,"gz")!=0&&comparecase(ext,"xz")!=0&&comparecase(ext,"lzma")!=0&&comparecase(ext,"bz2")!=0&&comparecase(ext,"zip")!=0&&comparecase(ext,"tgz")!=0&&comparecase(ext,"7z")!=0&&comparecase(ext,"tar")!=0){
				return 1;
			}
		}
	}
	sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
}



long ArchiveDialog::onCmdFileDelete(FXObject*,FXSelector,void*){
  FXint count=0;
  FXString logfiles,filetoremove,file;

  for(FXint i=0; i<filelist->getNumItems(); i++){
    if(filelist->isItemSelected(i)){
        file=filelist->getItemFilename(i);
        if(file=="..") continue;
	if(count>20) {logfiles+=".........................\n\nDelete more than 20 files?\n"; break;}
	logfiles+=file+"\n";
        count=count+1;
    }
  }

  FXuint answer=FXMessageBox::warning(this,MBOX_YES_NO,"Deleting Movie Files",logfiles.text());
  if(answer==MBOX_CLICKED_YES){

    for(FXint i=0; i<filelist->getNumItems(); i++){
      if(filelist->isItemSelected(i)){
        file=filelist->getItemFilename(i);
        if(file=="..") continue;
        filetoremove=FXPath::absolute(filelist->getDirectory(),file);
        FXTRACE((100,"filetoremove=%s\n",filetoremove.text()));
        if(!FXFile::removeFiles(filetoremove,TRUE)){
          if(MBOX_CLICKED_NO==FXMessageBox::error(this,MBOX_YES_NO,"Error Deleting Movie","Unable to delete movie: %s\nContinue with operation?",filetoremove.text())){
            break;
            }
          }
        }
      }
    }
  return 1;
  }


// Enable sender when Items have been selected
long ArchiveDialog::onUpdFileSelected(FXObject* sender,FXSelector,void*){
    sender->handle(this,getNumFilenames(filelist)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    return 1;
  }



// Get number of selected filenames, not including "." and ".."
FXint ArchiveDialog::getNumFilenames(FXFileList* filelist) const {
  register FXint num=0;
  register FXint i;
  if(filelist->getNumItems()){
    for(i=0; i<filelist->getNumItems(); i++){
#ifdef HAVE_LIBFOX_1_6
      if(filelist->isItemSelected(i) && filelist->getItemFilename(i)!=".." && filelist->getItemFilename(i)!=".") num++;
#endif
#ifdef HAVE_LIBFOX_1_7
      if(filelist->isItemSelected(i) && !filelist->isItemNavigational(i)) num++;
#endif
      }
    }
  return num;
  }



// Switch to given directory
void ArchiveDialog::setDirectory(const FXString& dirname){
  FXString newpath=FXPath::absolute(dirname);
  filelist->setDirectory(newpath);
  dirlist->setDirectory(newpath);
  }



// create application
void ArchiveDialog::create() {
    // Make file associations object;
    // shared between FXFileList and FXDirList
	if(!currentdir.empty()){
      dirlist->setDirectory(currentdir);
      filelist->setDirectory(currentdir);
    }

    if(!fontspec.empty()){
      bodyfont=new FXFont(getApp(),fontspec);
      filelist->setFont(bodyfont);
      dirlist->setFont(bodyfont);
    }

    if(!fontspec2.empty()){
      menufont=new FXFont(getApp(),fontspec2);
      getApp()->setNormalFont(menufont);
      button_ta->setFont(menufont);
      button_gz->setFont(menufont);
      button_bz->setFont(menufont);
      button_zi->setFont(menufont);
      button_7z->setFont(menufont);
      button_xz->setFont(menufont);
      button_ex->setFont(menufont);
      button_qu->setFont(menufont);
      title_arrange->setFont(menufont);
      radio_det->setFont(menufont);
      radio_sma->setFont(menufont);
      radio_big->setFont(menufont);
      title_show->setFont(menufont);
	  check_fil->setFont(menufont);
	  check_dir->setFont(menufont);
	  check_img->setFont(menufont);
      title_about->setFont(menufont);
	  cmd_about->setFont(menufont);
    }


    filelist->setHeaderIcon(0,m_renamediricon);
    filelist->setHeaderIcon(1,m_diricon);
    filelist->setHeaderIcon(2,m_dimicon);
    filelist->setHeaderIcon(3,m_daticon);
    filelist->setHeaderIcon(4,m_usricon);
    filelist->setHeaderIcon(5,m_grpicon);
    filelist->setHeaderIcon(6,m_prmicon);
    filelist->setHeaderIcon(7,m_consoleicon);


    associations=new FXFileDict(getApp());
    filelist->setAssociations(associations);
    dirlist->setAssociations(associations);

    FXMainWindow::create();
    show(PLACEMENT_SCREEN);

    return;
}



FXbool ArchiveDialog::readConfig() {
    // Read registry
	if(!getApp()->reg().read()){
		return false;
	}

    // Get size and position
#ifdef HAVE_LIBFOX_1_7
    FXuint width=getApp()->reg().readUIntEntry("FXArchive","width",700);
    FXuint height=getApp()->reg().readUIntEntry("FXArchive","height",400);
#endif
#ifdef HAVE_LIBFOX_1_6
    FXuint width=getApp()->reg().readUnsignedEntry("FXArchive","width",700);
    FXuint height=getApp()->reg().readUnsignedEntry("FXArchive","height",400);
#endif

    FXint xpos=getApp()->reg().readIntEntry("FXArchive","xpos",40);
    FXint ypos=getApp()->reg().readIntEntry("FXArchive","ypos",5);

    // Reposition window
    position(xpos,ypos,width,height);

    currentdir=getApp()->reg().readStringEntry("FXArchive","currentdir");

  	if(! (theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor"))) { theme_fxmmfore   = FXRGB(180,180,180);}
  	if(! (theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor"))) { theme_fxmmback   = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor"))) { theme_fxmmbase   = FXRGB( 60, 60, 60);}
  	if(! (theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor"))) { theme_fxmmborder = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor"))) { theme_fxmmmenufore  = FXRGB(240,240,240);}
  	if(! (theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor"))) { theme_fxmmmenuback  = FXRGB( 10, 10, 10);}
  	if(! (theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor"))) { theme_fxmmselfore   = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor"))) { theme_fxmmselback   = FXRGB(140,140,140);}
  	if(! (theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor"))) { theme_fxmmtipfore  = FXRGB(220,220,220);}
  	if(! (theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor"))) { theme_fxmmtipback  = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor"))) { theme_fxmmhilite  = FXRGB(100,100,100);}
  	if(! (theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor"))) { theme_fxmmshadow  = FXRGB( 10, 10, 10);}

  	getApp()->setForeColor(theme_fxmmfore);
  	getApp()->setBaseColor(theme_fxmmbase);
  	getApp()->setBackColor(theme_fxmmback);
  	getApp()->setBorderColor(theme_fxmmborder);
  	getApp()->setSelMenuTextColor(theme_fxmmmenufore);
  	getApp()->setSelMenuBackColor(theme_fxmmmenuback);
  	getApp()->setSelforeColor(theme_fxmmselfore);
  	getApp()->setSelbackColor(theme_fxmmselback);
  	getApp()->setTipforeColor(theme_fxmmtipfore);
  	getApp()->setTipbackColor(theme_fxmmtipback);
	getApp()->setHiliteColor(theme_fxmmhilite);
	getApp()->setShadowColor(theme_fxmmshadow);

    // Make font
    fontspec=getApp()->reg().readStringEntry("SETTINGS","bodyfont","");
    fontspec2=getApp()->reg().readStringEntry("SETTINGS","menufont","");

    terminal=getApp()->reg().readStringEntry("SETTINGS","terminal");

    // Read registry settings
	if(getApp()->reg().read()){
		return true;
	}
}



// About box
long ArchiveDialog::onCmdAbout(FXObject*,FXSelector,void*){
  FXMessageBox about(this,"About FXArchive",
	"FXArchive is a simple archive utility,\n"
	"Copyright (C) 2010 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA",
	NULL,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
	about.execute();
  return 1;
  }


// main
int main(int argc, char *argv[]) {
    // Make application
    FXApp application("Configuration","FXMM");
    // Open display
    application.init(argc,argv);
    // Make window: the new FXMovieManager(&application);
    ArchiveDialog* fxarchive = new ArchiveDialog(&application);
    // Handle interrupt to save stuff nicely
    application.addSignal(SIGINT,fxarchive,ArchiveDialog::ID_QUIT);
    // Create it
    application.create();
    // Start in given directory
    if(argc==2) fxarchive->setDirectory(argv[1]);
    // Run
    return application.run();
}


// EOF
