/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2009,2010 by Luigi Ziviani <ziviani_luigi@yahoo.it>.        *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: fxscanvirus.cpp,   v 1.2                   2008/01/01                     *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/
#include <fx.h>
#include "icons.h"
#include "fxscanvirus.h"
#include "config.h"
#ifdef HAVE_CLAMAV_H
#include <clamav.h>
#endif

// Message Map for the Scribble Window class
FXDEFMAP(ScanVirusDialog) ScanVirusDialogMap[]={

  //________Message_Type_____________________ID____________Message_Handler_______
  FXMAPFUNC(SEL_COMMAND,		   ScanVirusDialog::ID_QUIT,    ScanVirusDialog::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,		   ScanVirusDialog::ID_SCAN,    ScanVirusDialog::onCmdScan),

  FXMAPFUNC(SEL_DOUBLECLICKED, 	   ScanVirusDialog::ID_FILELIST,ScanVirusDialog::onCmdFilelist),
  FXMAPFUNC(SEL_COMMAND,		   ScanVirusDialog::ID_DIRLIST,	ScanVirusDialog::onCmdDirlist),

  FXMAPFUNC(SEL_COMMAND,		   ScanVirusDialog::ID_ABOUT,   ScanVirusDialog::onCmdAbout),
  };


// Macro for the ScanVirusApp class hierarchy implementation
FXIMPLEMENT(ScanVirusDialog,FXMainWindow,ScanVirusDialogMap,ARRAYNUMBER(ScanVirusDialogMap))

// Create properties dialog
ScanVirusDialog::ScanVirusDialog(FXApp *a)
  :FXMainWindow(a,"FX ScanVirus Utility",NULL,NULL,DECOR_ALL,0,0,680,600){

  // set icon on the main window
  logoicon=new FXPNGIcon(getApp(),fxsv64);
  logominiicon=new FXPNGIcon(getApp(),fxsv16);
  setIcon(logoicon);
  setMiniIcon(logominiicon);

  uicon=new FXGIFIcon(getApp(),m_folder_up);
  ricon=new FXGIFIcon(getApp(),m_folder_right);
  dicon=new FXGIFIcon(getApp(),m_folder_down);
  licon=new FXGIFIcon(getApp(),m_folder_left);

  cancelicon=new FXGIFIcon(getApp(),m_cancel);
  accepticon=new FXGIFIcon(getApp(),m_accept);
  infectedicon=new FXGIFIcon(getApp(),m_bug);
  scanicon=new FXGIFIcon(getApp(),m_bug_big);
  quiticon=new FXGIFIcon(getApp(),m_cancel_big);
  moveupicon=new FXGIFIcon(getApp(),m_up);

  if(!readConfig()){
	  printf("Configuration not loaded!");
  }

  // menubar
  menubar = new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

  dialogframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN, 0,0,0,0, 0,0,0,0,0,0);
  lineframe=new FXHorizontalFrame(dialogframe,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|FRAME_THICK,0,0,0,0, 0,0,0,0);
  label_cu=new FXLabel(lineframe,"Current File Scanned:");
  linelist=new FXTextField(lineframe,100,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0);

  splitterv=new FXSplitter(dialogframe,LAYOUT_SIDE_TOP|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_VERTICAL);
  viewframe=new FXHorizontalFrame(splitterv,LAYOUT_SIDE_BOTTOM|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  legendframe=new FXVerticalFrame(viewframe,LAYOUT_FILL_Y|FRAME_THICK, 0,0,0,0, 0,0,0,0,0,0);
  new FXLabel (legendframe," ");
  label_su=new FXLabel (legendframe," Summary:");
  new FXLabel (legendframe," ");
  label_cl=new FXLabel(legendframe,"Clean", accepticon);
  label_in=new FXLabel(legendframe,"Infected", infectedicon);
  label_re=new FXLabel(legendframe,"Removed", cancelicon);
  new FXLabel (legendframe," ");
  viewlist=new FXList(viewframe,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0);

  splitterh=new FXSplitter(splitterv,LAYOUT_SIDE_TOP|FRAME_RAISED|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_HORIZONTAL|SPLITTER_REVERSED);
  dirlist=new FXDirList(splitterh,this,ID_DIRLIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT|DIRLIST_NO_OWN_ASSOC,0,0,0,0);
  filelist=new FXFileList(splitterh,this,ID_FILELIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC,0,0,500,0);
  filelist->setDirectory(currentdir);
  dirlist->setDirectory(currentdir);

  // Arrange menu
  menu_arrange=new FXMenuPane(this);
  title_arrange=new FXMenuTitle(menubar,"&Arrange",NULL,menu_arrange);
  radio_det=new FXMenuRadio(menu_arrange,"Detailed &List\t\tView as detailed list.",filelist,FXFileList::ID_SHOW_DETAILS);
  radio_sma=new FXMenuRadio(menu_arrange,"&Small icons\t\tView as small icons.",filelist,FXFileList::ID_SHOW_MINI_ICONS);
  radio_big=new FXMenuRadio(menu_arrange,"&Big icons\t\tView as big icons.",filelist,FXFileList::ID_SHOW_BIG_ICONS);

  // Show/Hide menu
  menu_show=new FXMenuPane(this);
  title_show=new FXMenuTitle(menubar,"&Show/Hide",NULL,menu_show);
  check_fil=new FXMenuCheck(menu_show,"Hidden &Directories\t\tShow hidden directories.",dirlist,FXDirList::ID_TOGGLE_HIDDEN);
  check_dir=new FXMenuCheck(menu_show,"Hidden &Files\t\tShow hidden files and directories.",filelist,FXFileList::ID_TOGGLE_HIDDEN);

  // About menu
  menu_about=new FXMenuPane(this);
  title_about=new FXMenuTitle(menubar,"&About",NULL,menu_about,LAYOUT_RIGHT);
  cmd_about=new FXMenuCommand(menu_about,"&About\t\tAbout FXScanVirus",NULL,this,ID_ABOUT);

  // button to move up
  new FXButton(splitterh,"\tUp one level\tGo up to higher directory.",moveupicon,filelist,FXFileList::ID_DIRECTORY_UP,BUTTON_TOOLBAR|FRAME_RAISED|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_Y);


  new FXHorizontalSeparator(dialogframe,SEPARATOR_GROOVE|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);

  // Progress bar
  progressframe=new FXVerticalFrame(dialogframe,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0,0,0);
  progressbar=new FXProgressBar(progressframe,NULL,0,PROGRESSBAR_HORIZONTAL|PROGRESSBAR_PERCENTAGE|PROGRESSBAR_NORMAL|LAYOUT_FILL_X);
  totalprogressbar=new FXProgressBar(progressframe,NULL,0,PROGRESSBAR_HORIZONTAL|PROGRESSBAR_PERCENTAGE|PROGRESSBAR_NORMAL|LAYOUT_FILL_X);

  new FXHorizontalSeparator(dialogframe,SEPARATOR_GROOVE|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);

  // buttons
  buttonsframe=new FXHorizontalFrame(dialogframe,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0);
  button_sc=new FXButton(buttonsframe,tr(" &SCAN "),scanicon,this,ID_SCAN,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X,0,0,0,0,0,0,0,0);

  wheelframe=new FXHorizontalFrame(buttonsframe,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0, 0,0,0,0);
  label_wheel1=new FXLabel(wheelframe,"", uicon);
  label_wheel2=new FXLabel(wheelframe,"", ricon);
  label_wheel3=new FXLabel(wheelframe,"", dicon);
  label_wheel4=new FXLabel(wheelframe,"", licon);

  button_qu=new FXButton(buttonsframe,tr(" &QUIT "),quiticon,this,ID_QUIT,BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X,0,0,0,0,0,0,0,0);


  new FXSeparator(dialogframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXHorizontalFrame *copyright=new FXHorizontalFrame(dialogframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  FXLabel *label = new FXLabel(copyright,"FXScanVirus - Copyright (C) Luigi Ziviani",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();
  // set clamav to not-initialized
  initclamav=false;
}

// Destroy
ScanVirusDialog::~ScanVirusDialog(){
  delete accepticon;
  delete cancelicon;
  delete infectedicon;
  delete quiticon;
  delete scanicon;
  delete moveupicon;
  delete uicon;
  delete ricon;
  delete dicon;
  delete licon;
  }

void ScanVirusDialog::updateProgressionBars(FXint progress){
  progressbar->setProgress(0);
  progressbar->setTotal(progress);
  FXint stepprogress=totalprogress;
  totalprogress=totalprogress+progress;
  totalprogressbar->setTotal(totalprogress);
  totalprogressbar->setProgress(stepprogress);
}


void ScanVirusDialog::updateProgressionIcons(){
  FXIcon *wicon = NULL;
  wicon=label_wheel1->getIcon();

  if(wicon==uicon){
	label_wheel1->setIcon(ricon);	
	label_wheel2->setIcon(dicon);
	label_wheel3->setIcon(licon);
	label_wheel4->setIcon(uicon);	
  }else if(wicon==ricon){
	label_wheel1->setIcon(dicon);
	label_wheel2->setIcon(licon);
	label_wheel3->setIcon(uicon);
	label_wheel4->setIcon(ricon);	
  }else if(wicon==dicon){
	label_wheel1->setIcon(licon);
	label_wheel2->setIcon(uicon);
	label_wheel3->setIcon(ricon);	
	label_wheel4->setIcon(dicon);
  }else if(wicon==licon){
	label_wheel1->setIcon(uicon);
	label_wheel2->setIcon(ricon);
	label_wheel3->setIcon(dicon);
	label_wheel4->setIcon(licon);	
  }
  label_wheel1->repaint();
  label_wheel2->repaint();
  label_wheel3->repaint();
  label_wheel4->repaint();
}


int ScanVirusDialog::execScanVirus(FXFileList* panel, FXbool initclam){
    viewlist->clearItems();
#ifdef HAVE_LIBCLAMAV

    int ret;
    unsigned int sigs = 0;
    const char *virname;
    struct cl_engine *engine;

	getApp()->beginWaitCursor();

	// initialize clamav
	if (initclam==false){
		initclam=true;
    	if((ret = cl_init(CL_INIT_DEFAULT)) != CL_SUCCESS) {
      		FXMessageBox::error(this,MBOX_OK,"ERROR","Can't initialize libclamav: %s\n", cl_strerror(ret));
	  		return 1;
		}
    }
	// new engine
    if(!(engine = cl_engine_new())) {
      FXMessageBox::error(this,MBOX_OK,"ERROR","Can't create new engine");
      return 1;
    }
    // load all available databases from default directory
    if((ret = cl_load(cl_retdbdir(), engine, &sigs, CL_DB_STDOPT)) != CL_SUCCESS) {
      FXMessageBox::error(this,MBOX_OK,"Clamav_load: %s\n", cl_strerror(ret));
      cl_engine_free(engine);
      return 1;
    }

    printf("Loaded %u signatures.\n", sigs);

    // build engine
    if((ret = cl_engine_compile(engine)) != CL_SUCCESS) {
	  FXMessageBox::error(this,MBOX_OK,"ERROR","Database initialization error: %s\n", cl_strerror(ret));;
      cl_engine_free(engine);
	return 1;
    }

	getApp()->endWaitCursor();

    FXString filetoscan,file;
    progressbar->setBarColor(FXRGB(50,250,50));
    totalprogressbar->setBarColor(FXRGB(50,50,250));
	FXint progress;
    progress=0;
	totalprogress=0;
	// prevent sanning empty selection
    FXint selitem=panel->getCurrentItem();
    if(selitem<0){
		panel->setAnchorItem(0);
		FXint numitems=panel->getNumItems();
		panel->extendSelection(numitems-1);
    }

    // 0.0 manage files and directories
	progress=panel->getNumItems();
	updateProgressionBars(progress);
    for(FXint i=0; i<panel->getNumItems(); i++){
      if(panel->isItemSelected(i)){
        file=panel->getItemFilename(i);
        // 0.1 If it is a directory ...
        if(file=="..") { continue; }
        // 0.2 Else if it is a directory 
        else if(panel->isItemDirectory(i)){
	  	  FXString directory=panel->getItemPathname(i);
          FXuint answer=FXMessageBox::question(this,MBOX_YES_NO,"SCAN RECURSIVELY?","Directory: %s\n", directory.text());
	  	  if(answer==MBOX_CLICKED_YES){

	        getApp()->beginWaitCursor();

	        FXSplitter *splittern=new FXSplitter(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING|SPLITTER_VERTICAL|SPLITTER_REVERSED);
	        FXFileList *panel1=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	        panel1->setDirectory(directory);
  	        
            // 1.0 manage files and directories
			progress=panel1->getNumItems();
			updateProgressionBars(progress);
            for(FXint i=0; i<panel1->getNumItems(); i++){
              FXString file1=panel1->getItemFilename(i);
	      	  // 1.1	
              if(file1=="..") { continue; }
              // 1.2 Else if it is a directory 
              else if(panel1->isItemDirectory(i)){
				FXString directory1=panel1->getItemPathname(i);
	    		FXFileList *panel2=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    		panel2->setDirectory(directory1);
  	    		
                // 2.0 manage files and directories
				progress=panel2->getNumItems();
				updateProgressionBars(progress);
            	for(FXint i=0; i<panel2->getNumItems(); i++){
                  FXString file2=panel2->getItemFilename(i);
		  		  // 2.1
                  if(file2=="..") { continue; }
                  // 2.2 Else if it is a directory 
                  else if(panel2->isItemDirectory(i)){
		    	    FXString directory2=panel2->getItemPathname(i);
	    	    	FXFileList *panel3=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	panel3->setDirectory(directory2);
  	    	    	
                    // 3.0 manage files and directories
					progress=panel3->getNumItems();
					updateProgressionBars(progress);
            	    for(FXint i=0; i<panel3->getNumItems(); i++){
                      FXString file3=panel3->getItemFilename(i);
		        	  // 3.1
                      if(file3=="..") { continue; }
                  	  // 3.2 Else if it is a directory 
                  	  else if(panel3->isItemDirectory(i)){
		    	  	    FXString directory3=panel3->getItemPathname(i);
	    	    	  	FXFileList *panel4=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	  	panel4->setDirectory(directory3);
  	    	   	  	  	
                      	// 4.0 manage files and directories
						progress=panel4->getNumItems();
						updateProgressionBars(progress);
            	        for(FXint i=0; i<panel4->getNumItems(); i++){
                          FXString file4=panel4->getItemFilename(i);
		            	  // 4.1
                          if(file4=="..") { continue; }
		            	  // 4.2 Else if it is a directory
                  	      else if(panel4->isItemDirectory(i)){
		    	      	    FXString directory4=panel4->getItemPathname(i);
	    	    	        FXFileList *panel5=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	        panel5->setDirectory(directory4);
  	    	   	      	    
                      	    // 5.0 manage files and directories
							progress=panel5->getNumItems();
							updateProgressionBars(progress);
            	            for(FXint i=0; i<panel5->getNumItems(); i++){
                              FXString file5=panel5->getItemFilename(i);
		                      // 5.1
                              if(file5=="..") { continue; }
		                      // 5.2 Else if it is a directory
                  	          else if(panel5->isItemDirectory(i)){
		    	          	    FXString directory5=panel5->getItemPathname(i);
	    	    	            FXFileList *panel6=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	            panel6->setDirectory(directory5);
  	    	   	          	    
                      	  	    // 6.0 manage files and directories
								progress=panel6->getNumItems();
								updateProgressionBars(progress);
            	          	    for(FXint i=0; i<panel6->getNumItems(); i++){
                            	  FXString file6=panel6->getItemFilename(i);
		            	          // 6.1
                            	  if(file6=="..") { continue; }
		            	          // 6.2 Else if it is a directory
                  	    	      else if(panel6->isItemDirectory(i)){
		    	      	            FXString directory6=panel6->getItemPathname(i);
	    	    	     	        FXFileList *panel7=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	      	        panel7->setDirectory(directory6);
	    	   	      	            
               	  	  	      	    // 7.0 manage files and directories
									progress=panel7->getNumItems();
									updateProgressionBars(progress);
            	          	        for(FXint i=0; i<panel7->getNumItems(); i++){
                            	      FXString file7=panel7->getItemFilename(i);
		            	    		  // 7.1
                            	      if(file7=="..") { continue; }
									  // 7.2 Else if it is a directory
                  	    	          else if(panel7->isItemDirectory(i)){
		    	      	                FXString directory7=panel7->getItemPathname(i);
	    	    	     	            FXFileList *panel8=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	      	            panel8->setDirectory(directory7);
	    	   	      	            
               	  	  	      	        // 8.0 manage files and directories
										progress=panel8->getNumItems();
										updateProgressionBars(progress);
            	          	            for(FXint i=0; i<panel8->getNumItems(); i++){
                            	          FXString file8=panel8->getItemFilename(i);
		            	    		      // 8.1
                            	          if(file8=="..") { continue; }
									      // 8.2 Else if it is a directory
                  	    	              else if(panel8->isItemDirectory(i)){
		    	      	                    FXString directory8=panel8->getItemPathname(i);
	    	    	     	                FXFileList *panel9=new FXFileList(splittern,this,0,LAYOUT_FILL_X|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE|FILELIST_NO_OWN_ASSOC);
	    	    	      	                panel9->setDirectory(directory8);
	    	   	      	            
               	  	  	      	            // 9.0 manage files and directories
											progress=panel9->getNumItems();
											updateProgressionBars(progress);
            	          	                for(FXint i=0; i<panel9->getNumItems(); i++){
                            	              FXString file9=panel9->getItemFilename(i);
		            	    		          // 9.1
                            	              if(file9=="..") { continue; }
									          // 9.2 Else if it is a directory
                 	    	                  else if(panel9->isItemDirectory(i)){
		    	      	                        FXString directory9=panel9->getItemPathname(i);
					  				            FXMessageBox::error(this,MBOX_OK,"RECURSIVELY SCANNING FAILURE","Directory level exceded: \n", directory9.text());  	
									          }
								              // 9.3
		                                      else if(panel9->isItemFile(i)){
                                                filetoscan=FXPath::absolute(panel9->getDirectory(),file9);
			                                    scanFileName(ret, virname, engine, filetoscan, i);
		                                      }  
				      		                }  
  				    	                  }  
								          // 8.3
		                                  else if(panel8->isItemFile(i)){
                                            filetoscan=FXPath::absolute(panel8->getDirectory(),file8);
			                                scanFileName(ret, virname, engine, filetoscan, i);
		                                  }  
				      		            }  
  				    	              }  
								      // 7.3
		                              else if(panel7->isItemFile(i)){
                                        filetoscan=FXPath::absolute(panel7->getDirectory(),file7);
			                            scanFileName(ret, virname, engine, filetoscan, i);
		                              }  
				      		        }  
				    	          }  
			     	              // 6.3
		                          else if(panel6->isItemFile(i)){
                                    filetoscan=FXPath::absolute(panel6->getDirectory(),file6);
			                        scanFileName(ret, virname, engine, filetoscan, i);
		                          }
         	                    } 
                              }
			                  // 5.3
		                      else if(panel5->isItemFile(i)){
                                filetoscan=FXPath::absolute(panel5->getDirectory(),file5);
			                    scanFileName(ret, virname, engine, filetoscan, i);
		                      }
         	                }
                          }
			              // 4.3
		                  else if(panel4->isItemFile(i)){
                            filetoscan=FXPath::absolute(panel4->getDirectory(),file4);
			                scanFileName(ret, virname, engine, filetoscan, i);
		                  }
         	            }
                      }
			          // 3.3
		              else if(panel3->isItemFile(i)){
                        filetoscan=FXPath::absolute(panel3->getDirectory(),file3);
			            scanFileName(ret, virname, engine, filetoscan, i);
		              }
         	        }
                  }
		          // 2.3
		          else if(panel2->isItemFile(i)){
                    filetoscan=FXPath::absolute(panel2->getDirectory(),file2);
		            scanFileName(ret, virname, engine, filetoscan, i);
		          }
		        }
	          }
	          // 1.3
              else if(panel1->isItemFile(i)){
                filetoscan=FXPath::absolute(panel1->getDirectory(),file1);
		        scanFileName(ret, virname, engine, filetoscan, i);
	          }
	        }  
  	        FXuint answer=FXMessageBox::information(this,MBOX_OK_CANCEL,"Virus Scanning","No Virus Detected on %s\n", directory.text());
  	        if(answer==MBOX_CLICKED_CANCEL){
			  // free memory
	          cl_engine_free(engine);
	          return 1;
	        }
 	        directory=FXPath::upLevel(panel->getDirectory());
			updateProgressionBars(0);

	        getApp()->endWaitCursor();

          }
	  	  else if(answer==MBOX_CLICKED_NO){
			// free memory
			cl_engine_free(engine);
			return 1;
          }
        }
        // 0.3 Else if it is a regular file 
        else if(panel->isItemFile(i)){
          filetoscan=FXPath::absolute(panel->getDirectory(),file);
		  scanFileName(ret, virname, engine, filetoscan, i);
        }
        // 0.4 
      }
    }
    FXMessageBox::information(this,MBOX_OK,"Virus Scanning Summary","No know virus detected");
    // free memory
    cl_engine_free(engine);
#endif
    return 1;
  }


void ScanVirusDialog::scanFileName(int ret, const char *virname, struct cl_engine *engine, FXString& filetoscan, int i) {
#ifdef HAVE_LIBCLAMAV
    long double mb;
    unsigned long int size = 0;
	updateProgressionIcons();
	// show filename processed
	linelist->setText(filetoscan.text());
	linelist->repaint();
     // scan filename
     if((ret = cl_scanfile(filetoscan.text(), &virname, &size, engine, CL_SCAN_STDOPT)) == CL_VIRUS) {
    	progressbar->setBarColor(FXRGB(250,50,50));
   	    progressbar->increment(1);
   	    totalprogressbar->increment(1);
		viewlist->prependItem(filetoscan.text(), scanicon);
		viewlist->repaint();
        FXuint answer1=FXMessageBox::warning(this,MBOX_YES_NO,"Warning!","Virus detected: %s\nContinue with deleting dialog?", virname);
        // trying to delete the infected file
        if(answer1==MBOX_CLICKED_YES){
           FXuint answer2=FXMessageBox::warning(this,MBOX_YES_NO,"Deleting Movie!","Do YOU WANT TO DELETE this file? %s\n",filetoscan.text());
           if(answer2==MBOX_CLICKED_YES){
              FXTRACE((100,"filetoremove=%s\n",filetoscan.text()));
              if(!FXFile::remove(filetoscan)){
                if(MBOX_CLICKED_NO==FXMessageBox::error(this,MBOX_YES_NO,"Error Deleting Movie","Unable to delete movie: %s\nContinue with operation?",filetoscan.text())){
                   return;
                }
              }
			viewlist->prependItem(filetoscan.text(), cancelicon);
			viewlist->repaint();
            }
          }
        } else if(ret == CL_CLEAN) {
   	        progressbar->increment(1);
   	        totalprogressbar->increment(1);
			viewlist->prependItem(filetoscan.text(), accepticon);
			viewlist->repaint();
        } else {
          FXMessageBox::error(this,MBOX_OK,"ERROR","Error: %s\n", cl_strerror(ret));
          cl_engine_free(engine);
          return;
        }
     // calculate size of scanned data
     mb = size * (CL_COUNT_PRECISION / 1024) / 1024.0;
     printf("Data scanned: %2.2Lf MB\n", mb);
#endif
     return;
  }


// Scan at given positions 
long ScanVirusDialog::onCmdScan(FXObject*,FXSelector,void*){
  execScanVirus(filelist, initclamav);
  initclamav=true;
  return 1;
}


// Close dialog with quit
long ScanVirusDialog::onCmdQuit(FXObject*,FXSelector,void*){
  // Write new window size and position back to registry
#ifdef HAVE_LIBFOX_1_7
  getApp()->reg().writeUIntEntry("FXScanVirus","width",(FXuint)getWidth());
  getApp()->reg().writeUIntEntry("FXScanVirus","height",(FXuint)getHeight());
#endif
#ifdef HAVE_LIBFOX_1_6
  getApp()->reg().writeUnsignedEntry("FXScanVirus","width",(FXuint)getWidth());
  getApp()->reg().writeUnsignedEntry("FXScanVirus","height",(FXuint)getHeight());
#endif 
  getApp()->reg().writeIntEntry("FXScanVirus","xpos",getX());
  getApp()->reg().writeIntEntry("FXScanVirus","ypos",getY());
  // set currentdir
  currentdir = dirlist->getDirectory();
  getApp()->reg().writeStringEntry("FXScanVirus","currentdir",currentdir.text());
  // Write registry settings
  getApp()->reg().write();
  // Quit
  getApp()->exit(0);
}


// Command from the tree list
long ScanVirusDialog::onCmdFilelist(FXObject*,FXSelector,void* ptr){
  FXint index=(FXint)(FXival)ptr;

  if(0<=index){
    if(filelist->isItemDirectory(index)){
      currentdir=filelist->getItemPathname(index);
      filelist->setDirectory(currentdir);
      dirlist->setDirectory(currentdir);
      }
    else if(filelist->isItemFile(index)){
      filename=filelist->getItemPathname(index);
      execScanVirus(filelist, initclamav);
      initclamav=true;
      }
    }
  return 1;
}


// Change the directory from the FXDirList
long ScanVirusDialog::onCmdDirlist(FXObject*,FXSelector,void* ptr){
  FXTreeItem *item=(FXTreeItem*)ptr;
  FXString path=dirlist->getItemPathname(item);
  filelist->setDirectory(path);
  return 1;
  }


// Switch to given directory
void ScanVirusDialog::setDirectory(const FXString& dirname){
  FXString newpath=FXPath::absolute(dirname);
  filelist->setDirectory(newpath);
  dirlist->setDirectory(newpath);
  }


// create application
void ScanVirusDialog::create() {

    if(!fontspec.empty()){
      bodyfont=new FXFont(getApp(),fontspec);
      filelist->setFont(bodyfont);
      dirlist->setFont(bodyfont);
	  linelist->setFont(bodyfont);
    }

    if(!fontspec2.empty()){
      menufont=new FXFont(getApp(),fontspec2);
      getApp()->setNormalFont(menufont);
      viewlist->setFont(menufont);
      label_cu->setFont(menufont);
      label_su->setFont(menufont);
      label_cl->setFont(menufont);
      label_in->setFont(menufont);
      label_re->setFont(menufont);
      button_sc->setFont(menufont);
      button_qu->setFont(menufont);
      progressbar->setFont(menufont);
      totalprogressbar->setFont(menufont);
      title_arrange->setFont(menufont);
      radio_det->setFont(menufont);
      radio_sma->setFont(menufont);
      radio_big->setFont(menufont);
      title_show->setFont(menufont);
	  check_fil->setFont(menufont);
	  check_dir->setFont(menufont);
      title_about->setFont(menufont);
	  cmd_about->setFont(menufont);
    }

	if(!currentdir.empty()){
      dirlist->setDirectory(currentdir);
      filelist->setDirectory(currentdir);
    }

    // Make file associations object; 
    // shared between FXFileList and FXDirList
    associations=new FXFileDict(getApp());
    filelist->setAssociations(associations);
    dirlist->setAssociations(associations);

    FXMainWindow::create();
    show(PLACEMENT_SCREEN);

    return;
}

FXbool ScanVirusDialog::readConfig() {
    // Read registry
	if(!getApp()->reg().read()){
		return false;
	}

    // Get size and position
#ifdef HAVE_LIBFOX_1_7
    FXuint width=getApp()->reg().readUIntEntry("FXScanVirus","width",680);
    FXuint height=getApp()->reg().readUIntEntry("FXScanVirus","height",600);
#endif
#ifdef HAVE_LIBFOX_1_6
    FXuint width=getApp()->reg().readUnsignedEntry("FXScanVirus","width",680);
    FXuint height=getApp()->reg().readUnsignedEntry("FXScanVirus","height",600);
#endif 

    FXint xpos=getApp()->reg().readIntEntry("FXScanVirus","xpos",40);
    FXint ypos=getApp()->reg().readIntEntry("FXScanVirus","ypos",5);

    // Reposition window
    position(xpos,ypos,width,height);

    currentdir=getApp()->reg().readStringEntry("FXScanVirus","currentdir");

  	if(! (theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor"))) { theme_fxmmfore   = FXRGB(180,180,180);}
  	if(! (theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor"))) { theme_fxmmback   = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor"))) { theme_fxmmbase   = FXRGB( 60, 60, 60);}
  	if(! (theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor"))) { theme_fxmmborder = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor"))) { theme_fxmmmenufore  = FXRGB(240,240,240);}
  	if(! (theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor"))) { theme_fxmmmenuback  = FXRGB( 10, 10, 10);}
  	if(! (theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor"))) { theme_fxmmselfore   = FXRGB(  0,  0,  0);}
  	if(! (theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor"))) { theme_fxmmselback   = FXRGB(140,140,140);}
  	if(! (theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor"))) { theme_fxmmtipfore  = FXRGB(220,220,220);}
  	if(! (theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor"))) { theme_fxmmtipback  = FXRGB( 20, 20, 20);}
  	if(! (theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor"))) { theme_fxmmhilite  = FXRGB(100,100,100);}
  	if(! (theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor"))) { theme_fxmmshadow  = FXRGB( 10, 10, 10);}

  	getApp()->setForeColor(theme_fxmmfore);
  	getApp()->setBaseColor(theme_fxmmbase);
  	getApp()->setBackColor(theme_fxmmback);
  	getApp()->setBorderColor(theme_fxmmborder);
  	getApp()->setSelMenuTextColor(theme_fxmmmenufore);
  	getApp()->setSelMenuBackColor(theme_fxmmmenuback);
  	getApp()->setSelforeColor(theme_fxmmselfore);
  	getApp()->setSelbackColor(theme_fxmmselback);
  	getApp()->setTipforeColor(theme_fxmmtipfore);
  	getApp()->setTipbackColor(theme_fxmmtipback);
	getApp()->setHiliteColor(theme_fxmmhilite);
	getApp()->setShadowColor(theme_fxmmshadow);

    // Make font
    fontspec=getApp()->reg().readStringEntry("SETTINGS","bodyfont","");
    fontspec2=getApp()->reg().readStringEntry("SETTINGS","menufont","");

    // Read registry settings
	if(getApp()->reg().read()){
		return true;
	}
}


// About box
long ScanVirusDialog::onCmdAbout(FXObject*,FXSelector,void*){
  FXMessageBox about(this,"About FXArchive",
	"FXScanVirus is a simple virus scanner utility,\n"
	"Copyright (C) 2010 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA",
	NULL,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
	about.execute();
  return 1;
  }


// main
int main(int argc, char *argv[]) {
    // Make application
    FXApp application("Configuration","FXMM");
    // Open display
    application.init(argc,argv);
    // Make window: the new FXMovieManager(&application);
    ScanVirusDialog* fxscanvirus = new ScanVirusDialog(&application);
    // Handle interrupt to save stuff nicely
    application.addSignal(SIGINT,fxscanvirus,ScanVirusDialog::ID_QUIT);
    // Create it
    application.create();
    // Start in given directory
    if(argc==2) fxscanvirus->setDirectory(argv[1]);
    // Run
    return application.run();
}


// EOF
